# -*- coding: utf-8 -*-
'''
Support for Wireless Tools for Linux
'''

from __future__ import absolute_import
import logging
import salt.utils
from salt.exceptions import SaltInvocationError


log = logging.getLogger(__name__)


def __virtual__():
    '''
    Only load the module if iwconfig is installed
    '''
    if salt.utils.which('iwconfig'):
        return True
    return (False, 'The iwtools execution module cannot be loaded: '
            'iwconfig is not installed.')


def scan(iface, style=None):
    '''
    List networks on a wireless interface

    CLI Examples:

        salt minion iwtools.scan wlp3s0
        salt minion iwtools.scan wlp3s0 list
    '''
    if not _valid_iface(iface):
        raise SaltInvocationError(
            'The interface specified is not valid'
        )

    out = __salt__['cmd.run']('iwlist {0} scan'.format(iface))
    if 'Network is down' in out:
        __salt__['cmd.run']('ip link set {0} up'.format(iface))
        out = __salt__['cmd.run']('iwlist {0} scan'.format(iface))
    ret = {}
    tmp = {}
    for line in out.splitlines():
        if not line.strip():
            continue
        if 'Scan completed' in line:
            continue
        if line.strip().startswith('Cell'):
            comps = line.split(' - ')
            line = comps[1]
            if tmp:
                ret[tmp['Address']] = tmp
                tmp = {}
        comps = line.split(':')
        if comps[0].strip() == 'Address':
            # " is a valid character in SSIDs, but iwlist likes to wrap SSIDs in them
            comps[1] = comps[1].lstrip('"').rstrip('"')
        if comps[0].strip() == 'IE':
            if 'IE' not in tmp:
                tmp['IE'] = []
            tmp['IE'].append(':'.join(comps[1:]).strip())
        else:
            tmp[comps[0].strip()] = ':'.join(comps[1:]).strip()

    ret[tmp['Address']] = tmp

    if style == 'list':
        return ret.keys()

    return ret


def set_mode(iface, mode):
    '''
    List networks on a wireless interface

    CLI Example:

        salt minion iwtools.set_mode wlp3s0 Managed
    '''
    if not _valid_iface(iface):
        raise SaltInvocationError(
            'The interface specified is not valid'
        )

    valid_modes = ('Managed', 'Ad-Hoc', 'Master', 'Repeater', 'Secondary', 'Monitor', 'Auto')
    if mode not in valid_modes:
        raise SaltInvocationError(
            'One of the following modes must be specified: {0}'.format(', '.join(valid_modes))
        )
    __salt__['ip.down'](iface)
    out = __salt__['cmd.run']('iwconfig {0} mode {1}'.format(iface, mode))
    __salt__['ip.up'](iface)

    return mode


def _valid_iface(iface):
    '''
    Validate the specified interface
    '''
    ifaces = list_interfaces()
    if iface in ifaces.keys():
        return True
    return False


def list_interfaces(style=None):
    '''
    List all of the wireless interfaces

    CLI Example:

        salt minion iwtools.list_interfaces
    '''
    ret = {}
    tmp = None
    iface = None
    out = __salt__['cmd.run']('iwconfig')
    for line in out.splitlines():
        if not line:
            continue
        if 'no wireless extensions' in line:
            continue
        comps = line.strip().split('  ')
        if not line.startswith(' '):
            if tmp is not None:
                ret[iface] = tmp.copy()
            iface = comps.pop(0)
            tmp = {'extra': []}
        for item in comps:
            if ':' in item:
                parts = item.split(':')
                key = parts[0].strip()
                value = parts[1].strip()
                if key == 'ESSID':
                    value = value.lstrip('"').rstrip('"')
                tmp[key] = value
            elif '=' in item:
                parts = item.split('=')
                tmp[parts[0].strip()] = parts[1].strip()
            else:
                tmp['extra'].append(item)

    ret[iface] = tmp.copy()

    if style == 'list':
        return ret.keys()

    return ret



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rahul Handay <rahulha@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import user

# Globals
user.__salt__ = {}
user.__opts__ = {}
user.__grains__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class UserTestCase(TestCase):
    '''
        Validate the user state
    '''
    def test_present(self):
        '''
            Test to ensure that the named user is present with
            the specified properties
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': ''}
        mock = MagicMock(return_value=False)
        mock2 = MagicMock(return_value=[])
        with patch.dict(user.__salt__, {'group.info': mock,
                                        'user.info': mock2,
                                        "user.chkey": mock2,
                                        'user.add': mock}):
            ret.update({'comment': 'The following group(s) are'
                        ' not present: salt'})
            self.assertDictEqual(user.present('salt', groups=['salt']), ret)

            mock = MagicMock(side_effect=[{'key': 'value'}, {'key': 'value'},
                                          {'key': 'value'}, False, False])
            with patch.object(user, '_changes', mock):
                with patch.dict(user.__opts__, {"test": True}):
                    ret.update({'comment': 'The following user attributes are'
                                ' set to be changed:\nkey: value\n',
                                'result': None})
                    self.assertDictEqual(user.present('salt'), ret)

                with patch.dict(user.__opts__, {"test": False}):
                    with patch.dict(user.__grains__, {"kernel": False}):
                        ret.update({'comment': "These values could not be"
                                    " changed: {'key': 'value'}",
                                    'result': False})
                        self.assertDictEqual(user.present('salt'), ret)

                        with patch.dict(user.__opts__, {"test": True}):
                            ret.update({'comment': 'User salt set to'
                                        ' be added', 'result': None})
                            self.assertDictEqual(user.present('salt'), ret)

                        with patch.dict(user.__opts__, {"test": False}):
                            ret.update({'comment': 'Failed to create new'
                                        ' user salt', 'result': False})
                            self.assertDictEqual(user.present('salt'), ret)

    def test_absent(self):
        '''
            Test to ensure that the named user is absent
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': None,
               'comment': ''}
        mock = MagicMock(side_effect=[True, True, False])
        mock1 = MagicMock(return_value=False)
        with patch.dict(user.__salt__, {'user.info': mock,
                                        'user.delete': mock1,
                                        'group.info': mock1}):
            with patch.dict(user.__opts__, {"test": True}):
                ret.update({'comment': 'User salt set for removal'})
                self.assertDictEqual(user.absent('salt'), ret)

            with patch.dict(user.__opts__, {"test": False}):
                ret.update({'comment': 'Failed to remove user salt',
                            'result': False})
                self.assertDictEqual(user.absent('salt'), ret)

            ret.update({'comment': 'User salt is not present',
                        'result': True})
            self.assertDictEqual(user.absent('salt'), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(UserTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Microsoft certificate management via the Pki PowerShell module.

:platform:      Windows

.. versionadded:: 2016.11.0
'''

# Import python libs
from __future__ import absolute_import

_DEFAULT_CONTEXT = 'LocalMachine'
_DEFAULT_FORMAT = 'cer'
_DEFAULT_STORE = 'My'


def __virtual__():
    '''
    Load only on minions that have the win_pki module.
    '''
    if 'win_pki.get_stores' in __salt__:
        return True
    return False


def import_cert(name, cert_format=_DEFAULT_FORMAT, context=_DEFAULT_CONTEXT, store=_DEFAULT_STORE,
                exportable=True, password='', saltenv='base'):
    '''
    Import the certificate file into the given certificate store.

    :param str name: The path of the certificate file to import.
    :param str cert_format: The certificate format. Specify 'cer' for X.509, or 'pfx' for PKCS #12.
    :param str context: The name of the certificate store location context.
    :param str store: The name of the certificate store.
    :param bool exportable: Mark the certificate as exportable. Only applicable to pfx format.
    :param str password: The password of the certificate. Only applicable to pfx format.
    :param str saltenv: The environment the file resides in.

    Example of usage with only the required arguments:

    .. code-block:: yaml

        site0-cert-imported:
            win_pki.import_cert:
                - name: salt://win/webserver/certs/site0.cer

    Example of usage specifying all available arguments:

    .. code-block:: yaml

        site0-cert-imported:
            win_pki.import_cert:
                - name: salt://win/webserver/certs/site0.pfx
                - cert_format: pfx
                - context: LocalMachine
                - store: My
                - exportable: True
                - password: TestPassword
                - saltenv: base
    '''
    ret = {'name': name,
           'changes': dict(),
           'comment': str(),
           'result': None}

    store_path = r'Cert:\{0}\{1}'.format(context, store)

    cached_source_path = __salt__['cp.cache_file'](name, saltenv)
    current_certs = __salt__['win_pki.get_certs'](context=context, store=store)
    cert_props = __salt__['win_pki.get_cert_file'](name=cached_source_path)

    if cert_props['thumbprint'] in current_certs:
        ret['comment'] = ("Certificate '{0}' already contained in store:"
                          ' {1}').format(cert_props['thumbprint'], store_path)
        ret['result'] = True
    elif __opts__['test']:
        ret['comment'] = ("Certificate '{0}' will be imported into store:"
                          ' {1}').format(cert_props['thumbprint'], store_path)
        ret['changes'] = {'old': None,
                          'new': cert_props['thumbprint']}
    else:
        ret['changes'] = {'old': None,
                          'new': cert_props['thumbprint']}
        ret['result'] = __salt__['win_pki.import_cert'](name=name, cert_format=cert_format,
                                                        context=context, store=store,
                                                        exportable=exportable, password=password,
                                                        saltenv=saltenv)
        if ret['result']:
            ret['comment'] = ("Certificate '{0}' imported into store:"
                              ' {1}').format(cert_props['thumbprint'], store_path)
        else:
            ret['comment'] = ("Certificate '{0}' unable to be imported into store:"
                              ' {1}').format(cert_props['thumbprint'], store_path)
    return ret


def remove_cert(name, thumbprint, context=_DEFAULT_CONTEXT, store=_DEFAULT_STORE):
    '''
    Remove the certificate from the given certificate store.

    :param str thumbprint: The thumbprint value of the target certificate.
    :param str context: The name of the certificate store location context.
    :param str store: The name of the certificate store.

    Example of usage with only the required arguments:

    .. code-block:: yaml

        site0-cert-removed:
            win_pki.remove_cert:
                - thumbprint: 9988776655443322111000AAABBBCCCDDDEEEFFF

    Example of usage specifying all available arguments:

    .. code-block:: yaml

        site0-cert-removed:
            win_pki.remove_cert:
                - thumbprint: 9988776655443322111000AAABBBCCCDDDEEEFFF
                - context: LocalMachine
                - store: My
    '''
    ret = {'name': name,
           'changes': dict(),
           'comment': str(),
           'result': None}

    store_path = r'Cert:\{0}\{1}'.format(context, store)
    current_certs = __salt__['win_pki.get_certs'](context=context, store=store)

    if thumbprint not in current_certs:
        ret['comment'] = "Certificate '{0}' already removed from store: {1}".format(thumbprint,
                                                                                    store_path)
        ret['result'] = True
    elif __opts__['test']:
        ret['comment'] = "Certificate '{0}' will be removed from store: {1}".format(thumbprint,
                                                                                    store_path)
        ret['changes'] = {'old': thumbprint,
                          'new': None}
    else:
        ret['changes'] = {'old': thumbprint,
                          'new': None}
        ret['result'] = __salt__['win_pki.remove_cert'](thumbprint=thumbprint, context=context,
                                                        store=store)
        if ret['result']:
            ret['comment'] = "Certificate '{0}' removed from store: {1}".format(thumbprint, store_path)
        else:
            ret['comment'] = "Certificate '{0}' unable to be removed from store: {1}".format(thumbprint,
                                                                                             store_path)
    return ret



# -*- coding: utf-8 -*-
'''
Support for getting and setting the environment variables
of the current salt process.
'''
from __future__ import absolute_import

# Import salt libs
import salt.utils as utils

# Import python libs
import os
import logging

# Import 3rd-party libs
import salt.ext.six as six

log = logging.getLogger(__name__)


def __virtual__():
    '''
    No dependency checks, and not renaming, just return True
    '''
    return True


def setval(key, val, false_unsets=False, permanent=False):
    '''
    Set a single salt process environment variable. Returns True
    on success.

    key
        The environment key to set. Must be a string.

    val
        The value to set. Must be a string or False. Refer to the
        'false_unsets' parameter for behavior when set to False.

    false_unsets
        If val is False and false_unsets is True, then the key will be
        removed from the salt processes environment dict entirely.
        If val is False and false_unsets is not True, then the key's
        value will be set to an empty string.
        Default: False.

    permanent
        On Windows minions this will set the environment variable in the
        registry so that it is always added as a environment variable when
        applications open. If you want to set the variable to HKLM instead of
        HKCU just pass in "HKLM" for this parameter. On all other minion types
        this will be ignored. Note: This will only take affect on applications
        opened after this has been set.

    CLI Example:

    .. code-block:: bash

        salt '*' environ.setval foo bar
        salt '*' environ.setval baz val=False false_unsets=True
        salt '*' environ.setval baz bar permanent=True
        salt '*' environ.setval baz bar permanent=HKLM
    '''
    is_windows = utils.is_windows()
    if is_windows:
        permanent_hive = 'HKCU'
        permanent_key = 'Environment'
        if permanent == 'HKLM':
            permanent_hive = 'HKLM'
            permanent_key = r'SYSTEM\CurrentControlSet\Control\Session Manager\Environment'

    if not isinstance(key, six.string_types):
        log.debug(
            '{0}: \'key\' argument is not a string type: \'{1}\''
            .format(__name__, key)
        )
    if val is False:
        if false_unsets is True:
            try:
                os.environ.pop(key, None)
                if permanent and is_windows:
                    __salt__['reg.delete_value'](permanent_hive, permanent_key, key)
                return None
            except Exception as exc:
                log.error(
                    '{0}: Exception occurred when unsetting '
                    'environ key \'{1}\': \'{2}\''
                    .format(__name__, key, exc)
                )
                return False
        else:
            val = ''
    if isinstance(val, six.string_types):
        try:
            os.environ[key] = val
            if permanent and is_windows:
                __salt__['reg.set_value'](permanent_hive, permanent_key, key, val)
            return os.environ[key]
        except Exception as exc:
            log.error(
                '{0}: Exception occurred when setting'
                'environ key \'{1}\': \'{2}\''
                .format(__name__, key, exc)
            )
            return False
    else:
        log.debug(
            '{0}: \'val\' argument for key \'{1}\' is not a string '
            'or False: \'{2}\''
            .format(__name__, key, val)
        )
        return False


def setenv(environ, false_unsets=False, clear_all=False, update_minion=False, permanent=False):
    '''
    Set multiple salt process environment variables from a dict.
    Returns a dict.

    environ
        Must be a dict. The top-level keys of the dict are the names
        of the environment variables to set. Each key's value must be
        a string or False. Refer to the 'false_unsets' parameter for
        behavior when a value set to False.

    false_unsets
        If a key's value is False and false_unsets is True, then the
        key will be removed from the salt processes environment dict
        entirely. If a key's value is False and false_unsets is not
        True, then the key's value will be set to an empty string.
        Default: False

    clear_all
        USE WITH CAUTION! This option can unset environment variables
        needed for salt to function properly.
        If clear_all is True, then any environment variables not
        defined in the environ dict will be deleted.
        Default: False

    update_minion
        If True, apply these environ changes to the main salt-minion
        process. If False, the environ changes will only affect the
        current salt subprocess.
        Default: False

    permanent
        On Windows minions this will set the environment variable in the
        registry so that it is always added as a environment variable when
        applications open. If you want to set the variable to HKLM instead of
        HKCU just pass in "HKLM" for this parameter. On all other minion types
        this will be ignored. Note: This will only take affect on applications
        opened after this has been set.


    CLI Example:

    .. code-block:: bash

        salt '*' environ.setenv '{"foo": "bar", "baz": "quux"}'
        salt '*' environ.setenv '{"a": "b", "c": False}' false_unsets=True
    '''
    ret = {}
    if not isinstance(environ, dict):
        log.debug(
            '{0}: \'environ\' argument is not a dict: \'{1}\''
            .format(__name__, environ)
        )
        return False
    if clear_all is True:
        # Unset any keys not defined in 'environ' dict supplied by user
        to_unset = [key for key in os.environ if key not in environ]
        for key in to_unset:
            ret[key] = setval(key, False, false_unsets, permanent=permanent)
    for key, val in six.iteritems(environ):
        if isinstance(val, six.string_types):
            ret[key] = setval(key, val, permanent=permanent)
        elif val is False:
            ret[key] = setval(key, val, false_unsets, permanent=permanent)
        else:
            log.debug(
                '{0}: \'val\' argument for key \'{1}\' is not a string '
                'or False: \'{2}\''
                .format(__name__, key, val)
            )
            return False

    if update_minion is True:
        __salt__['event.fire']({'environ': environ,
                                'false_unsets': false_unsets,
                                'clear_all': clear_all,
                                'permanent': permanent
                                },
                               'environ_setenv')
    return ret


def get(key, default=''):
    '''
    Get a single salt process environment variable.

    key
        String used as the key for environment lookup.

    default
        If the key is not found in the environment, return this value.
        Default: ''


    CLI Example:

    .. code-block:: bash

        salt '*' environ.get foo
        salt '*' environ.get baz default=False
    '''
    if not isinstance(key, six.string_types):
        log.debug(
            '{0}: \'key\' argument is not a string type: \'{1}\''
            .format(__name__, key)
        )
        return False
    return os.environ.get(key, default)


def has_value(key, value=None):
    '''
    Determine whether the key exists in the current salt process
    environment dictionary. Optionally compare the current value
    of the environment against the supplied value string.

    key
        Must be a string. Used as key for environment lookup.

    value:
        Optional. If key exists in the environment, compare the
        current value with this value. Return True if they are equal.

    CLI Example:

    .. code-block:: bash

        salt '*' environ.has_value foo
    '''
    if not isinstance(key, six.string_types):
        log.debug(
            '{0}: \'key\' argument is not a string type: \'{1}\''
            .format(__name__, key)
        )
        return False
    try:
        cur_val = os.environ[key]
        if value is not None:
            if cur_val == value:
                return True
            else:
                return False
    except KeyError:
        return False
    return True


def item(keys, default=''):
    '''
    Get one or more salt process environment variables.
    Returns a dict.

    keys
        Either a string or a list of strings that will be used as the
        keys for environment lookup.

    default
        If the key is not found in the environment, return this value.
        Default: ''

    CLI Example:

    .. code-block:: bash

        salt '*' environ.item foo
        salt '*' environ.item '[foo, baz]' default=None
    '''
    ret = {}
    key_list = []
    if isinstance(keys, six.string_types):
        key_list.append(keys)
    elif isinstance(keys, list):
        key_list = keys
    else:
        log.debug(
            '{0}: \'keys\' argument is not a string or list type: \'{1}\''
            .format(__name__, keys)
        )
    for key in key_list:
        ret[key] = os.environ.get(key, default)
    return ret


def items():
    '''
    Return a dict of the entire environment set for the salt process

    CLI Example:

    .. code-block:: bash

        salt '*' environ.items
    '''
    return dict(os.environ)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import rdp

rdp.__opts__ = {}
rdp.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class RdpTestCase(TestCase):
    '''
    Test cases for salt.states.rdp
    '''
    # 'enabled' function tests: 1

    def test_enabled(self):
        '''
        Test to enable the RDP service and make sure access
        to the RDP port is allowed in the firewall configuration.
        '''
        name = 'my_service'

        ret = {'name': name,
               'changes': {},
               'result': True,
               'comment': ''}

        mock_t = MagicMock(side_effect=[False, False, True])
        mock_f = MagicMock(return_value=False)
        with patch.dict(rdp.__salt__,
                        {'rdp.status': mock_t,
                         'rdp.enable': mock_f}):
            with patch.dict(rdp.__opts__, {'test': True}):
                comt = ('RDP will be enabled')
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(rdp.enabled(name), ret)

            with patch.dict(rdp.__opts__, {'test': False}):
                ret.update({'comment': '', 'result': False,
                            'changes': {'RDP was enabled': True}})
                self.assertDictEqual(rdp.enabled(name), ret)

                comt = ('RDP is enabled')
                ret.update({'comment': comt, 'result': True,
                            'changes': {}})
                self.assertDictEqual(rdp.enabled(name), ret)

    # 'disabled' function tests: 1

    def test_disabled(self):
        '''
        Test to disable the RDP service.
        '''
        name = 'my_service'

        ret = {'name': name,
               'changes': {},
               'result': True,
               'comment': ''}

        mock = MagicMock(side_effect=[True, True, False])
        mock_t = MagicMock(return_value=True)
        with patch.dict(rdp.__salt__,
                        {'rdp.status': mock,
                         'rdp.disable': mock_t}):
            with patch.dict(rdp.__opts__, {'test': True}):
                comt = ('RDP will be disabled')
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(rdp.disabled(name), ret)

            with patch.dict(rdp.__opts__, {'test': False}):
                ret.update({'comment': '', 'result': True,
                            'changes': {'RDP was disabled': True}})
                self.assertDictEqual(rdp.disabled(name), ret)

                comt = ('RDP is disabled')
                ret.update({'comment': comt, 'result': True, 'changes': {}})
                self.assertDictEqual(rdp.disabled(name), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(RdpTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email: `Bo Maryniuk <bo@suse.de>`
'''

# Import Python libs
from __future__ import absolute_import
import errno
from mock import Mock

# Import Salt Testing libs
from salttesting import TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import patch
from salt.ext.six.moves import range

ensure_in_syspath('../')

from salt.fileclient import Client


class FileclientTestCase(TestCase):
    '''
    Fileclient test
    '''
    opts = {
        'extension_modules': '',
        'cachedir': '/__test__',
    }

    def _fake_makedir(self, num=errno.EEXIST):
        def _side_effect(*args, **kwargs):
            raise OSError(num, 'Errno {0}'.format(num))
        return Mock(side_effect=_side_effect)

    def test_cache_skips_makedirs_on_race_condition(self):
        '''
        If cache contains already a directory, do not raise an exception.
        '''
        with patch('os.path.isfile', lambda prm: False):
            for exists in range(2):
                with patch('os.makedirs', self._fake_makedir()):
                    with Client(self.opts)._cache_loc('testfile') as c_ref_itr:
                        assert c_ref_itr == '/__test__/files/base/testfile'

    def test_cache_raises_exception_on_non_eexist_ioerror(self):
        '''
        If makedirs raises other than EEXIST errno, an exception should be raised.
        '''
        with patch('os.path.isfile', lambda prm: False):
            with patch('os.makedirs', self._fake_makedir(num=errno.EROFS)):
                with self.assertRaises(OSError):
                    with Client(self.opts)._cache_loc('testfile') as c_ref_itr:
                        assert c_ref_itr == '/__test__/files/base/testfile'



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# committer_parser.py
#
# Simple script to parse the output of 'git log' and generate some statistics.
# May leverage GitHub API in the future
#
'''
To use this commit parser script pipe git log into the stdin:

    git log | committer_parser.py -c -
'''

# Import python libs
from __future__ import absolute_import
from __future__ import print_function
import sys
import getopt
import re
import email.utils
import datetime


class Usage(Exception):
    def __init__(self, msg):  # pylint: disable=W0231
        self.msg = 'committer_parser.py [-c | --contributor-detail] - |' \
                    ' <logfilename>\n'
        self.msg += '   : Parse commit log from git and print number of ' \
                    'commits and unique committers\n'
        self.msg += '   : by month.  Accepts a filename or reads from stdin.\n'
        self.msg += '   : -c | --contributor-detail generates output by ' \
                    'contributor, by month, in a tab-separated table\n'
        if msg:
            self.msg += '\n'
            self.msg += msg


def parse_date(datestr):
    d = email.utils.parsedate(datestr)
    return datetime.datetime(d[0], d[1], d[2], d[3], d[4], d[5], d[6])


def parse_gitlog(filename=None):
    '''
    Parse out the gitlog cli data
    '''
    results = {}
    commits = {}
    commits_by_contributor = {}

    if not filename or filename == '-':
        fh = sys.stdin
    else:
        fh = open(filename, 'r+')

    commitcount = 0
    for line in fh.readlines():
        line = line.rstrip()
        if line.startswith('commit '):
            new_commit = True
            commitcount += 1
            continue

        if line.startswith('Author:'):
            author = re.match(r'Author:\s+(.*)\s+<(.*)>', line)
            if author:
                email = author.group(2)
            continue

        if line.startswith('Date:'):

            isodate = re.match(r'Date:\s+(.*)', line)
            d = parse_date(isodate.group(1))
            continue

        if len(line) < 2 and new_commit:
            new_commit = False
            key = '{0}-{1}'.format(d.year, str(d.month).zfill(2))

            if key not in results:
                results[key] = []

            if key not in commits:
                commits[key] = 0

            if email not in commits_by_contributor:
                commits_by_contributor[email] = {}

            if key not in commits_by_contributor[email]:
                commits_by_contributor[email][key] = 1
            else:
                commits_by_contributor[email][key] += 1

                if email not in results[key]:
                    results[key].append(email)

                commits[key] += commitcount
                commitcount = 0
    fh.close()
    return (results, commits, commits_by_contributor)


def counts_by_contributor(commits_by_contributor, results):
    output = ''
    dates = sorted(results.keys())
    for d in dates:
        output += '\t{0}'.format(d)

    output += '\n'

    for email in sorted(commits_by_contributor.keys()):
        output += '\'{0}'.format(email)
        for d in dates:
            if d in commits_by_contributor[email]:
                output += '\t{0}'.format(commits_by_contributor[email][d])
            else:
                output += '\t'
        output += '\n'
    return output


def count_results(results, commits):
    result_str = ''
    print('Date\tContributors\tCommits')
    for k in sorted(results.keys()):
        result_str += '{0}\t{1}\t{2}'.format(k, len(results[k]), commits[k])
        result_str += '\n'
    return result_str


def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(argv[1:], 'hc', ['help', 'contributor-detail'])
            if len(args) < 1:
                raise Usage('committer_parser.py needs a filename or \'-\' to read from stdin')
        except getopt.error as msg:
            raise Usage(msg)
    except Usage as err:
        print(err.msg, file=sys.stderr)
        return 2

    if len(opts) > 0:
        if '-h' in opts[0] or '--help' in opts[0]:
            return 0

    data, counts, commits_by_contributor = parse_gitlog(filename=args[0])

    if len(opts) > 0:
        if '-c' or '--contributor-detail':
            print(counts_by_contributor(commits_by_contributor, data))
    else:
        print(count_results(data, counts))

if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Justin Anderson <janderson@saltstack.com>`
'''

# Python Libs
from __future__ import absolute_import
import os

# Salt Libs
from salt.modules import beacons
from salt.exceptions import CommandExecutionError
import integration

# Salttesting libs
from salttesting import skipIf
from salttesting.helpers import destructiveTest, ensure_in_syspath


ensure_in_syspath('../../')

beacons.__opts__ = {}


class BeaconsAddDeleteTest(integration.ModuleCase):
    '''
    Tests the add and delete functions
    '''
    def setUp(self):
        self.minion_conf_d_dir = os.path.join(
                self.minion_opts['config_dir'],
                os.path.dirname(self.minion_opts['default_include']))
        if not os.path.isdir(self.minion_conf_d_dir):
            os.makedirs(self.minion_conf_d_dir)
        self.beacons_config_file_path = os.path.join(self.minion_conf_d_dir, 'beacons.conf')

    def tearDown(self):
        if os.path.isfile(self.beacons_config_file_path):
            os.unlink(self.beacons_config_file_path)

    def test_add_and_delete(self):
        '''
        Test adding and deleting a beacon
        '''
        _add = self.run_function('beacons.add', ['ps', [{'apache2': 'stopped'}]])
        self.assertTrue(_add['result'])

        # save added beacon
        _save = self.run_function('beacons.save')
        self.assertTrue(_save['result'])

        # delete the beacon
        _delete = self.run_function('beacons.delete', ['ps'])
        self.assertTrue(_delete['result'])

        # save the results
        self.run_function('beacons.save')


@destructiveTest
class BeaconsTest(integration.ModuleCase):
    '''
    Tests the beacons execution module
    '''
    beacons_config_file_path = minion_conf_d_dir = None

    @classmethod
    def tearDownClass(cls):
        if os.path.isfile(cls.beacons_config_file_path):
            os.unlink(cls.beacons_config_file_path)

    def setUp(self):
        if self.minion_conf_d_dir is None:
            self.minion_conf_d_dir = os.path.join(
                    self.minion_opts['config_dir'],
                    os.path.dirname(self.minion_opts['default_include']))
            if not os.path.isdir(self.minion_conf_d_dir):
                os.makedirs(self.minion_conf_d_dir)
        self.__class__.beacons_config_file_path = os.path.join(self.minion_conf_d_dir, 'beacons.conf')
        try:
            # Add beacon to disable
            self.run_function('beacons.add', ['ps', [{'apache2': 'stopped'}]])
            self.run_function('beacons.save')
        except CommandExecutionError:
            self.skipTest('Unable to add beacon')

    def tearDown(self):
        # delete added beacon
        self.run_function('beacons.delete', ['ps'])
        self.run_function('beacons.save')

    def test_disable(self):
        '''
        Test disabling beacons
        '''
        # assert beacon exists
        _list = self.run_function('beacons.list', return_yaml=False)
        self.assertIn('ps', _list)

        ret = self.run_function('beacons.disable')
        self.assertTrue(ret['result'])

        # assert beacons are disabled
        _list = self.run_function('beacons.list', return_yaml=False)
        self.assertFalse(_list['enabled'])

        # disable added beacon
        ret = self.run_function('beacons.disable_beacon', ['ps'])
        self.assertTrue(ret['result'])

        # assert beacon ps is disabled
        _list = self.run_function('beacons.list', return_yaml=False)
        for bdict in _list['ps']:
            if 'enabled' in bdict:
                self.assertFalse(bdict['enabled'])
                break

    def test_enable(self):
        '''
        Test enabling beacons
        '''
        # assert beacon exists
        _list = self.run_function('beacons.list', return_yaml=False)
        self.assertIn('ps', _list)

        # enable beacons on minion
        ret = self.run_function('beacons.enable')
        self.assertTrue(ret['result'])

        # assert beacons are enabled
        _list = self.run_function('beacons.list', return_yaml=False)
        self.assertTrue(_list['enabled'])

    @skipIf(True, 'Skip until https://github.com/saltstack/salt/issues/31516 problems are resolved.')
    def test_enabled_beacons(self):
        '''
        Test enabled specific beacon
        '''
        # enable added beacon
        ret = self.run_function('beacons.enable_beacon', ['ps'])
        self.assertTrue(ret['result'])

        # assert beacon ps is enabled
        _list = self.run_function('beacons.list', return_yaml=False)
        self.assertTrue(_list['ps']['enabled'])

    def test_list(self):
        '''
        Test lising the beacons
        '''
        # list beacons
        ret = self.run_function('beacons.list', return_yaml=False)
        if 'enabled' in ret:
            self.assertEqual(ret, {'ps': [{'apache2': 'stopped'}], 'enabled': True})
        else:
            self.assertEqual(ret, {'ps': {'apache': 'stopped'}})


if __name__ == '__main__':
    from integration import run_tests
    run_tests([BeaconsAddDeleteTest, BeaconsTest])



# -*- coding: utf-8 -*-
'''
Tests for the file state
'''
# Import python libs
from __future__ import absolute_import
import errno
import os
import textwrap
import tempfile

# Import Salt Testing libs
from salttesting.helpers import ensure_in_syspath
ensure_in_syspath('../../')

# Import salt libs
import integration
import salt.utils

IS_WINDOWS = salt.utils.is_windows()
STATE_DIR = os.path.join(integration.FILES, 'file', 'base')


class CMDTest(integration.ModuleCase,
              integration.SaltReturnAssertsMixIn):
    '''
    Validate the cmd state
    '''
    def test_run_simple(self):
        '''
        cmd.run
        '''
        cmd = 'dir' if IS_WINDOWS else 'ls'
        ret = self.run_state('cmd.run', name=cmd, cwd=tempfile.gettempdir())
        self.assertSaltTrueReturn(ret)

    def test_test_run_simple(self):
        '''
        cmd.run test interface
        '''
        ret = self.run_state('cmd.run', name='ls',
                             cwd=tempfile.gettempdir(), test=True)
        self.assertSaltNoneReturn(ret)


class CMDRunRedirectTest(integration.ModuleCase,
                         integration.SaltReturnAssertsMixIn):
    '''
    Validate the cmd state of run_redirect
    '''
    def setUp(self):
        self.state_name = 'run_redirect'
        state_filename = self.state_name + '.sls'
        self.state_file = os.path.join(STATE_DIR, state_filename)

        # Create the testfile and release the handle
        self.fd, self.test_file = tempfile.mkstemp()
        try:
            os.close(self.fd)
        except OSError as exc:
            if exc.errno != errno.EBADF:
                raise exc

        super(CMDRunRedirectTest, self).setUp()

    def tearDown(self):
        try:
            os.remove(self.state_file)
            os.remove(self.test_file)
        except OSError:
            # Not all of the tests leave files around that we want to remove
            # As some of the tests create the sls files in the test itself,
            # And some are using files in the integration test file state tree.
            pass
        super(CMDRunRedirectTest, self).tearDown()

    def test_run_unless(self):
        '''
        test cmd.run unless
        '''
        state_key = 'cmd_|-/var/log/messages_|-/var/log/messages_|-run'
        with salt.utils.fopen(self.state_file, 'w') as fb_:
            fb_.write(textwrap.dedent('''
                /var/log/messages:
                  cmd.run:
                    - unless: echo cheese > {0}
                '''.format(self.test_file)))

        ret = self.run_function('state.sls', [self.state_name])
        self.assertTrue(ret[state_key]['result'])

    def test_run_unless_multiple_cmds(self):
        '''
        test cmd.run using multiple unless options where the first cmd in the
        list will pass, but the second will fail. This tests the fix for issue
        #35384. (The fix is in PR #35545.)
        '''
        sls = self.run_function('state.sls', mods='issue-35384')
        self.assertSaltTrueReturn(sls)
        # We must assert against the comment here to make sure the comment reads that the
        # command "echo "hello"" was run. This ensures that we made it to the last unless
        # command in the state. If the comment reads "unless execution succeeded", or similar,
        # then the unless state run bailed out after the first unless command succeeded,
        # which is the bug we're regression testing for.
        self.assertEqual(sls['cmd_|-cmd_run_unless_multiple_|-echo "hello"_|-run']['comment'],
                         'Command "echo "hello"" run')

    def test_run_creates_exists(self):
        '''
        test cmd.run creates already there
        '''
        state_key = 'cmd_|-echo >> {0}_|-echo >> {0}_|-run'.format(self.test_file)
        with salt.utils.fopen(self.state_file, 'w') as fb_:
            fb_.write(textwrap.dedent('''
                echo >> {0}:
                  cmd.run:
                    - creates: {0}
                '''.format(self.test_file)))

        ret = self.run_function('state.sls', [self.state_name])
        self.assertTrue(ret[state_key]['result'])
        self.assertEqual(len(ret[state_key]['changes']), 0)

    def test_run_creates_new(self):
        '''
        test cmd.run creates not there
        '''
        os.remove(self.test_file)
        state_key = 'cmd_|-echo >> {0}_|-echo >> {0}_|-run'.format(self.test_file)
        with salt.utils.fopen(self.state_file, 'w') as fb_:
            fb_.write(textwrap.dedent('''
                echo >> {0}:
                  cmd.run:
                    - creates: {0}
                '''.format(self.test_file)))

        ret = self.run_function('state.sls', [self.state_name])
        self.assertTrue(ret[state_key]['result'])
        self.assertEqual(len(ret[state_key]['changes']), 4)

    def test_run_redirect(self):
        '''
        test cmd.run with shell redirect
        '''
        state_key = 'cmd_|-echo test > {0}_|-echo test > {0}_|-run'.format(self.test_file)
        with salt.utils.fopen(self.state_file, 'w') as fb_:
            fb_.write(textwrap.dedent('''
                echo test > {0}:
                  cmd.run
                '''.format(self.test_file)))

        ret = self.run_function('state.sls', [self.state_name])
        self.assertTrue(ret[state_key]['result'])


class CMDRunWatchTest(integration.ModuleCase,
                      integration.SaltReturnAssertsMixIn):
    '''
    Validate the cmd state of run_watch
    '''
    def setUp(self):
        self.state_name = 'run_watch'
        state_filename = self.state_name + '.sls'
        self.state_file = os.path.join(STATE_DIR, state_filename)
        super(CMDRunWatchTest, self).setUp()

    def tearDown(self):
        os.remove(self.state_file)
        super(CMDRunWatchTest, self).tearDown()

    def test_run_watch(self):
        '''
        test cmd.run watch
        '''
        saltines_key = 'cmd_|-saltines_|-echo changed=true_|-run'
        biscuits_key = 'cmd_|-biscuits_|-echo biscuits_|-wait'

        with salt.utils.fopen(self.state_file, 'w') as fb_:
            fb_.write(textwrap.dedent('''
                saltines:
                  cmd.run:
                    - name: echo changed=true
                    - cwd: /
                    - stateful: True

                biscuits:
                  cmd.wait:
                    - name: echo biscuits
                    - cwd: /
                    - watch:
                        - cmd: saltines
                '''))

        ret = self.run_function('state.sls', [self.state_name])
        self.assertTrue(ret[saltines_key]['result'])
        self.assertTrue(ret[biscuits_key]['result'])


if __name__ == '__main__':
    from integration import run_tests
    run_tests([CMDTest, CMDRunRedirectTest, CMDRunWatchTest])



# -*- coding: utf-8 -*-
'''
Module for managing Infoblox

Will look for pillar data infoblox:server, infoblox:user, infoblox:password if not passed to functions

.. versionadded:: 2016.3.0

:depends:
        - requests
'''
from __future__ import absolute_import

# Import salt libs
from salt.exceptions import CommandExecutionError
from salt.exceptions import SaltInvocationError
import logging

log = logging.getLogger(__name__)

try:
    import json
    import requests
    HAS_IMPORTS = True
except ImportError:
    HAS_IMPORTS = False


def __virtual__():
    if HAS_IMPORTS:
        return True
    return (False, 'The infoblox execution module cannot be loaded: '
            'python requests and/or json libraries are not available.')


def _conn_info_check(infoblox_server=None,
                     infoblox_user=None,
                     infoblox_password=None):
    '''
    get infoblox stuff from pillar if not passed
    '''

    if infoblox_server is None:
        infoblox_server = __salt__['pillar.get']('infoblox:server', None)
    if infoblox_user is None:
        infoblox_user = __salt__['pillar.get']('infoblox:user', None)
        log.debug('Infoblox username is "{0}"'.format(infoblox_user))
    if infoblox_password is None:
        infoblox_password = __salt__['pillar.get']('infoblox:password', None)

    return infoblox_server, infoblox_user, infoblox_password


def _process_return_data(retData):
    '''
    generic return processing
    '''
    if retData.status_code == 200:
        if retData.json():
            return retData
        else:
            log.debug('no data returned from infoblox')
            return None
    else:
        msg = 'Unsuccessful error code {0} returned'.format(retData.status_code)
        raise CommandExecutionError(msg)
    return None


def delete_record(name,
                  dns_view,
                  record_type,
                  infoblox_server=None,
                  infoblox_user=None,
                  infoblox_password=None,
                  infoblox_api_version='v1.4.2',
                  sslVerify=True):
    '''
    delete a record

    name
        name of the record

    dns_view
        the DNS view to remove the record from

    record_type
        the record type (a, cname, host, etc)

    infoblox_server
        the infoblox server hostname (can also use the infoblox:server pillar)

    infoblox_user
        the infoblox user to connect with (can also use the infoblox:user pillar)

    infoblox_password
        the infoblox user's password (can also use the infolblox:password pillar)

    infoblox_api_version
        the infoblox api version to use

    sslVerify
        should ssl verification be done on the connection to the Infoblox REST API

    CLI Example:

    .. code-block:: bash

        salt my-minion infoblox.delete_record some.dns.record MyInfobloxView A sslVerify=False
    '''
    infoblox_server, infoblox_user, infoblox_password = _conn_info_check(infoblox_server,
                                                                         infoblox_user,
                                                                         infoblox_password)
    if infoblox_server is None and infoblox_user is None and infoblox_password is None:
        _throw_no_creds()
        return None

    record_type = record_type.lower()
    currentRecords = get_record(name,
                                record_type,
                                infoblox_server,
                                infoblox_user,
                                infoblox_password,
                                dns_view,
                                infoblox_api_version,
                                sslVerify)
    if currentRecords:
        for currentRecord in currentRecords:
            url = 'https://{0}/wapi/{1}/{2}'.format(infoblox_server,
                                                    infoblox_api_version,
                                                    currentRecord['Record ID'])
            ret = requests.delete(url,
                                  auth=(infoblox_user, infoblox_password),
                                  headers={'Content-Type': 'application/json'},
                                  verify=sslVerify)
            if ret.status_code == 200:
                return True
            else:
                msg = 'Unsuccessful error code {0} returned -- full json dump {1}'.format(ret.status_code, ret.json())
                raise CommandExecutionError(msg)
    return False


def update_record(name,
                  value,
                  dns_view,
                  record_type,
                  infoblox_server=None,
                  infoblox_user=None,
                  infoblox_password=None,
                  infoblox_api_version='v1.4.2',
                  sslVerify=True):
    '''
    update an entry to an infoblox dns view

    name
        the dns name

    value
        the value for the record

    record_type
        the record type (a, cname, etc)

    dns_view
        the DNS view to add the record to

    infoblox_server
        the infoblox server hostname (can also use the infoblox:server pillar)

    infoblox_user
        the infoblox user to connect with (can also use the infoblox:user pillar)

    infoblox_password
        the infoblox user's password (can also use the infolblox:password pillar)

    infoblox_api_version
        the infoblox api version to use

    sslVerify
        should ssl verification be done on the connection to the Infoblox REST API

    CLI Example:

    .. code-block:: bash

        salt '*' infoblox.update_record alias.network.name canonical.network.name MyInfobloxView cname sslVerify=False
    '''

    infoblox_server, infoblox_user, infoblox_password = _conn_info_check(infoblox_server,
                                                                         infoblox_user,
                                                                         infoblox_password)
    if infoblox_server is None and infoblox_user is None and infoblox_password is None:
        _throw_no_creds()
        return None

    record_type = record_type.lower()
    currentRecords = get_record(name,
                                record_type,
                                infoblox_server,
                                infoblox_user,
                                infoblox_password,
                                dns_view,
                                infoblox_api_version,
                                sslVerify)
    if currentRecords:
        for currentRecord in currentRecords:
            url = 'https://{0}/wapi/{1}/{2}'.format(
                infoblox_server,
                infoblox_api_version,
                currentRecord['Record ID'])
            data = None
            if record_type == 'cname':
                data = json.dumps({'canonical': value})
            elif record_type == 'a':
                data = json.dumps({'ipv4addr': value})
            elif record_type == 'host':
                data = {'ipv4addrs': []}
                for i in value:
                    data['ipv4addrs'].append({'ipv4addr': i})
                data = json.dumps(data)
            ret = requests.put(url,
                               data,
                               auth=(infoblox_user, infoblox_password),
                               headers={'Content-Type': 'application/json'},
                               verify=sslVerify)
            if ret.status_code == 200:
                return True
            else:
                msg = 'Unsuccessful status code {0} returned.'.format(ret.status_code)
                raise CommandExecutionError(msg)
    else:
        msg = 'Record {0} of type {1} was not found'.format(name, record_type)
        log.error(msg)
        return False


def add_record(name,
               value,
               record_type,
               dns_view,
               infoblox_server=None,
               infoblox_user=None,
               infoblox_password=None,
               infoblox_api_version='v1.4.2',
               sslVerify=True):
    '''
    add a record to an infoblox dns view

    name
        the record name

    value
        the value for the entry
            can make use of infoblox functions for next available IP, like 'func:nextavailableip:10.1.0.0/24'

    record_type
        the record type (cname, a, host, etc)

    dns_view
        the DNS view to add the record to

    infoblox_server
        the infoblox server hostname (can also use the infoblox:server pillar)

    infoblox_user
        the infoblox user to connect with (can also use the infoblox:user pillar)

    infoblox_password
        the infoblox user's password (can also use the infolblox:password pillar)

    infoblox_api_version
        the infoblox api version to use

    sslVerify
        should ssl verification be done on the connection to the Infoblox REST API

    CLI Example:

    .. code-block:: bash

        salt 'myminion' infoblox.add_record alias.network.name canonical.network.name MyView
    '''

    infoblox_server, infoblox_user, infoblox_password = _conn_info_check(infoblox_server,
                                                                         infoblox_user,
                                                                         infoblox_password)
    if infoblox_server is None and infoblox_user is None and infoblox_password is None:
        _throw_no_creds()
        return None

    record_type = record_type.lower()

    data = None
    url = None
    if record_type == 'cname':
        data = json.dumps({'name': name, 'canonical': value, 'view': dns_view})
        log.debug('cname data {0}'.format(data))
    elif record_type == 'host':
        data = json.dumps({'name': name, 'ipv4addrs': [{'ipv4addr': value}], 'view': dns_view})
        log.debug('host record data {0}'.format(data))
    elif record_type == 'a':
        data = json.dumps({'name': name, 'ipv4addr': value, 'view': dns_view})
        log.debug('a record data {0}'.format(data))

    url = 'https://{0}/wapi/{1}/record:{2}'.format(infoblox_server,
                                                   infoblox_api_version,
                                                   record_type)

    ret = requests.post(url,
                        data,
                        auth=(infoblox_user, infoblox_password),
                        headers={'Content-Type': 'application/json'},
                        verify=sslVerify)
    if ret.status_code == 201:
        return True
    else:
        msg = 'Unsuccessful error code {0} returned -- full json dump {1}'.format(ret.status_code, ret.json())
        raise CommandExecutionError(msg)


def _throw_no_creds():
    '''
    helper function to log no credentials found error
    '''
    msg = 'An infoblox server, username, and password must be specified or configured via pillar'
    raise SaltInvocationError(msg)


def get_network(network_name,
                network_view=None,
                infoblox_server=None,
                infoblox_user=None,
                infoblox_password=None,
                infoblox_api_version='v1.4.2',
                sslVerify=True):
    '''
    get a network from infoblox

    network_name
        The name of the network in IPAM

    network_view
        The name of the network view the network belongs to

    infoblox_server
        the infoblox server hostname (can also use the infoblox:server pillar)

    infoblox_user
        the infoblox user to connect with (can also use the infoblox:user pillar)

    infoblox_password
        the infoblox user's password (can also use the infolblox:password pillar)

    infoblox_api_version
        the infoblox api version to use

    sslVerify
        should ssl verification be done on the connection to the Infoblox REST API

    CLI Example:

    .. code-block:: bash

        salt myminion infoblox.get_network '10.0.0.0/8'
    '''

    records = []
    infoblox_server, infoblox_user, infoblox_password = _conn_info_check(infoblox_server,
                                                                         infoblox_user,
                                                                         infoblox_password)
    if infoblox_server is None and infoblox_user is None and infoblox_password is None:
        _throw_no_creds()
        return None

    url = 'https://{0}/wapi/{1}/network?network={2}{3}'.format(
        infoblox_server,
        infoblox_api_version,
        network_name,
        ('' if network_view is None else '&network_view=' + network_view))
    log.debug('Requst url is "{0}"'.format(url))
    ret = _process_return_data(requests.get(url,
                                            auth=(infoblox_user, infoblox_password),
                                            verify=sslVerify))
    if ret:
        for entry in ret.json():
            log.debug('Infoblox record returned: {0}'.format(entry))
            tEntry = {}
            data = _parse_record_data(entry)
            for key in data.keys():
                tEntry[key] = data[key]
            records.append(tEntry)
        return records
    else:
        return False
    return False


def get_record(record_name,
               record_type='host',
               infoblox_server=None,
               infoblox_user=None,
               infoblox_password=None,
               dns_view=None,
               infoblox_api_version='v1.4.2',
               sslVerify=True):
    '''
    get a record from infoblox

    record_name
        name of the record to search for

    record_type
        type of reacord to search for (host, cname, a, etc...defaults to host)

    infoblox_server
        the infoblox server hostname (can also use the infoblox:server pillar)

    infoblox_user
        the infoblox user to connect with (can also use the infoblox:user pillar)

    infoblox_password
        the infoblox user's password (can also use the infolblox:password pillar)

    dns_view
        the infoblox DNS view to search, if not specified all views are searched

    infoblox_api_version
        the infoblox api version to use

    sslVerify
        should ssl verification be done on the connection to the Infoblox REST API

    CLI Example:

    .. code-block:: bash

        salt myminion infoblox.get_record some.host.com A sslVerify=False
    '''

    # TODO - verify record type (A, AAAA, CNAME< HOST, MX, PTR, SVR, TXT, host_ipv4addr, host_ipv6addr, naptr)
    records = []

    infoblox_server, infoblox_user, infoblox_password = _conn_info_check(infoblox_server,
                                                                         infoblox_user,
                                                                         infoblox_password)

    record_type = record_type.lower()
    if infoblox_server is None and infoblox_user is None and infoblox_password is None:
        _throw_no_creds()
        return None

    url = 'https://{0}/wapi/{1}/record:{3}?name:={2}{4}{5}'.format(
        infoblox_server,
        infoblox_api_version,
        record_name,
        record_type,
        ('' if dns_view is None else '&view=' + dns_view),
        ('&_return_fields%2B=aliases' if record_type == 'host' else '')
        )
    log.debug('Requst url is "{0}"'.format(url))
    ret = _process_return_data(requests.get(url,
                                            auth=(infoblox_user, infoblox_password),
                                            verify=sslVerify))
    if ret:
        for entry in ret.json():
            log.debug('Infoblox record returned: {0}'.format(entry))
            tEntry = {}
            data = _parse_record_data(entry)
            for key in data.keys():
                tEntry[key] = data[key]
            records.append(tEntry)
        return records
    else:
        return False
    return False


def _parse_record_data(entry_data):
    '''
    returns the right value data we'd be interested in for the specified record type
    '''

    ret = {}
    ipv4addrs = []
    aliases = []
    if 'canonical' in entry_data:
        ret['Canonical Name'] = entry_data['canonical']
    if 'ipv4addrs' in entry_data:
        for ipaddrs in entry_data['ipv4addrs']:
            ipv4addrs.append(ipaddrs['ipv4addr'])
        ret['IP Addresses'] = ipv4addrs
    if 'ipv4addr' in entry_data:
        ret['IP Address'] = entry_data['ipv4addr']
    if 'aliases' in entry_data:
        for alias in entry_data['aliases']:
            aliases.append(alias)
        ret['Aliases'] = aliases
    if 'name' in entry_data:
        ret['Name'] = entry_data['name']
    if 'view' in entry_data:
        ret['DNS View'] = entry_data['view']
    if 'network_view' in entry_data:
        ret['Network View'] = entry_data['network_view']
    if 'comment' in entry_data:
        ret['Comment'] = entry_data['comment']
    if 'network' in entry_data:
        ret['Network'] = entry_data['network']
    if '_ref' in entry_data:
        ret['Record ID'] = entry_data['_ref']
    return ret



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rahul Handay <rahulha@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Libs
from salt.states import host

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

ensure_in_syspath('../../')

host.__salt__ = {}
host.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class HostTestCase(TestCase):
    '''
        Validate the host state
    '''
    def test_present(self):
        '''
            Test to ensures that the named host is present with the given ip
        '''
        ret = {'changes': {},
               'comment': 'Host salt (127.0.0.1) already present',
               'name': 'salt', 'result': True}

        mock = MagicMock(return_value=True)
        with patch.dict(host.__salt__, {'hosts.has_pair': mock}):
            self.assertDictEqual(host.present("salt", "127.0.0.1"), ret)

    def test_absent(self):
        '''
            Test to ensure that the named host is absent
        '''
        ret = {'changes': {},
               'comment': 'Host salt (127.0.0.1) already absent',
               'name': 'salt', 'result': True}

        mock = MagicMock(return_value=False)
        with patch.dict(host.__salt__, {'hosts.has_pair': mock}):
            self.assertDictEqual(host.absent("salt", "127.0.0.1"), ret)

    def test_only_already(self):
        '''
        Test only() when the state hasn't changed
        '''
        expected = {
            'name': '127.0.1.1',
            'changes': {},
            'result': True,
            'comment': 'IP address 127.0.1.1 already set to "foo.bar"'}
        mock1 = MagicMock(return_value=['foo.bar'])
        with patch.dict(host.__salt__, {'hosts.get_alias': mock1}):
            mock2 = MagicMock(return_value=True)
            with patch.dict(host.__salt__, {'hosts.set_host': mock2}):
                with patch.dict(host.__opts__, {'test': False}):
                    self.assertDictEqual(
                        expected,
                        host.only("127.0.1.1", 'foo.bar'))

    def test_only_dryrun(self):
        '''
        Test only() when state would change, but it's a dry run
        '''
        expected = {
            'name': '127.0.1.1',
            'changes': {},
            'result': None,
            'comment': 'Would change 127.0.1.1 from "foo.bar" to "foo.bar foo"'}
        mock1 = MagicMock(return_value=['foo.bar'])
        with patch.dict(host.__salt__, {'hosts.get_alias': mock1}):
            mock2 = MagicMock(return_value=True)
            with patch.dict(host.__salt__, {'hosts.set_host': mock2}):
                with patch.dict(host.__opts__, {'test': True}):
                    self.assertDictEqual(
                        expected,
                        host.only("127.0.1.1", ['foo.bar', 'foo']))

    def test_only_fail(self):
        '''
        Test only() when state change fails
        '''
        expected = {
            'name': '127.0.1.1',
            'changes': {},
            'result': False,
            'comment': 'hosts.set_host failed to change 127.0.1.1'
                + ' from "foo.bar" to "foo.bar foo"'}
        mock = MagicMock(return_value=['foo.bar'])
        with patch.dict(host.__salt__, {'hosts.get_alias': mock}):
            mock = MagicMock(return_value=False)
            with patch.dict(host.__salt__, {'hosts.set_host': mock}):
                with patch.dict(host.__opts__, {'test': False}):
                    self.assertDictEqual(
                        expected,
                        host.only("127.0.1.1", ['foo.bar', 'foo']))

    def test_only_success(self):
        '''
        Test only() when state successfully changes
        '''
        expected = {
            'name': '127.0.1.1',
            'changes': {'127.0.1.1': {'old': 'foo.bar', 'new': 'foo.bar foo'}},
            'result': True,
            'comment': 'successfully changed 127.0.1.1'
                + ' from "foo.bar" to "foo.bar foo"'}
        mock = MagicMock(return_value=['foo.bar'])
        with patch.dict(host.__salt__, {'hosts.get_alias': mock}):
            mock = MagicMock(return_value=True)
            with patch.dict(host.__salt__, {'hosts.set_host': mock}):
                with patch.dict(host.__opts__, {'test': False}):
                    self.assertDictEqual(
                        expected,
                        host.only("127.0.1.1", ['foo.bar', 'foo']))


if __name__ == '__main__':
    from integration import run_tests
    run_tests(HostTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Common functions for working with RPM packages
'''

# Import python libs
from __future__ import absolute_import
import collections
import logging

# Import salt libs
from salt._compat import subprocess

log = logging.getLogger(__name__)

# These arches compiled from the rpmUtils.arch python module source
ARCHES_64 = ('x86_64', 'athlon', 'amd64', 'ia32e', 'ia64', 'geode')
ARCHES_32 = ('i386', 'i486', 'i586', 'i686')
ARCHES_PPC = ('ppc', 'ppc64', 'ppc64iseries', 'ppc64pseries')
ARCHES_S390 = ('s390', 's390x')
ARCHES_SPARC = (
    'sparc', 'sparcv8', 'sparcv9', 'sparcv9v', 'sparc64', 'sparc64v'
)
ARCHES_ALPHA = (
    'alpha', 'alphaev4', 'alphaev45', 'alphaev5', 'alphaev56',
    'alphapca56', 'alphaev6', 'alphaev67', 'alphaev68', 'alphaev7'
)
ARCHES_ARM = ('armv5tel', 'armv5tejl', 'armv6l', 'armv7l')
ARCHES_SH = ('sh3', 'sh4', 'sh4a')

ARCHES = ARCHES_64 + ARCHES_32 + ARCHES_PPC + ARCHES_S390 + \
    ARCHES_ALPHA + ARCHES_ARM + ARCHES_SH

# EPOCHNUM can't be used until RHEL5 is EOL as it is not present
QUERYFORMAT = '%{NAME}_|-%{EPOCH}_|-%{VERSION}_|-%{RELEASE}_|-%{ARCH}_|-%{REPOID}'


def get_osarch():
    '''
    Get the os architecture using rpm --eval
    '''
    ret = subprocess.Popen(
        'rpm --eval "%{_host_cpu}"',
        shell=True,
        close_fds=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE).communicate()[0]
    return ret or 'unknown'


def check_32(arch, osarch=None):
    '''
    Returns True if both the OS arch and the passed arch are 32-bit
    '''
    if osarch is None:
        osarch = get_osarch()
    return all(x in ARCHES_32 for x in (osarch, arch))


def pkginfo(name, version, arch, repoid):
    '''
    Build and return a pkginfo namedtuple
    '''
    pkginfo_tuple = collections.namedtuple(
        'PkgInfo',
        ('name', 'version', 'arch', 'repoid')
    )
    return pkginfo_tuple(name, version, arch, repoid)


def resolve_name(name, arch, osarch=None):
    '''
    Resolve the package name and arch into a unique name referred to by salt.
    For example, on a 64-bit OS, a 32-bit package will be pkgname.i386.
    '''
    if osarch is None:
        osarch = get_osarch()

    if not check_32(arch, osarch) and arch not in (osarch, 'noarch'):
        name += '.{0}'.format(arch)
    return name


def parse_pkginfo(line, osarch=None):
    '''
    A small helper to parse an rpm/repoquery command's output. Returns a
    pkginfo namedtuple.
    '''
    try:
        name, epoch, version, release, arch, repoid = line.split('_|-')
    # Handle unpack errors (should never happen with the queryformat we are
    # using, but can't hurt to be careful).
    except ValueError:
        return None

    name = resolve_name(name, arch, osarch)
    if release:
        version += '-{0}'.format(release)
    if epoch not in ('(none)', '0'):
        version = ':'.join((epoch, version))

    return pkginfo(name, version, arch, repoid)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

import inspect

# Import Salt Libs
from salt.modules import defaults

# Globals
defaults.__grains__ = {}
defaults.__salt__ = {}
defaults.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DefaultsTestCase(TestCase):
    '''
    Test cases for salt.modules.defaults
    '''
    @patch('salt.modules.defaults.get',
           MagicMock(return_value={'users': {'root': [0]}}))
    def test_get_mock(self):
        '''
        Test if it execute a defaults client run and return a dict
        '''
        with patch.object(inspect, 'stack', MagicMock(return_value=[])):
            self.assertEqual(defaults.get('core:users:root'),
                             {'users': {'root': [0]}})


if __name__ == '__main__':
    from integration import run_tests
    run_tests(DefaultsTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)
from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import memcached
from salt.exceptions import CommandExecutionError, SaltInvocationError
from salt.ext.six import integer_types


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MemcachedTestCase(TestCase):
    '''
    Test cases for salt.modules.memcached
    '''
    # 'status' function tests: 2

    def test_status(self):
        '''
        Test if it gets memcached status
        '''
        class MockMemcache(object):
            """
            Mock of memcache
            """
            @staticmethod
            def get_stats():
                """
                Mock of stats method
                """
                return [('127.0.0.1:11211 (1)', {})]

        with patch.object(memcached, '_connect',
                          MagicMock(return_value=MockMemcache())):
            self.assertDictEqual(memcached.status(),
                                 {'127.0.0.1:11211 (1)': {}})

    def test_status_false(self):
        '''
        Test if it gets memcached status
        '''
        class MockMemcache(object):
            """
            Mock of memcache
            """
            @staticmethod
            def get_stats():
                """
                Mock of stats method
                """
                return []

        with patch.object(memcached, '_connect',
                          MagicMock(return_value=MockMemcache())):
            self.assertFalse(memcached.status())

    # 'get' function tests: 1

    def test_get(self):
        '''
        Test if it retrieve value for a key
        '''
        class MockMemcache(object):
            """
            Mock of memcache
            """
            @staticmethod
            def get_stats():
                """
                Mock of stats method
                """
                return [('127.0.0.1:11211 (1)', {})]

            @staticmethod
            def get(key):
                """
                Mock of get method
                """
                return key

        with patch.object(memcached, '_connect',
                          MagicMock(return_value=MockMemcache())):
            self.assertEqual(memcached.get('salt'), 'salt')

    # 'set_' function tests: 1

    def test_set(self):
        '''
        Test if it set a key on the memcached server
        '''
        class MockMemcache(object):
            """
            Mock of memcache
            """
            def __init__(self):
                self.key = None
                self.value = None
                self.time = None
                self.min_compress_len = None

            @staticmethod
            def get_stats():
                """
                Mock of stats method
                """
                return [('127.0.0.1:11211 (1)', {})]

            def set(self, key, value, time, min_compress_len):
                """
                Mock of set method
                """
                self.key = key
                self.value = value
                self.time = time
                self.min_compress_len = min_compress_len
                return True

        with patch.object(memcached, '_connect',
                          MagicMock(return_value=MockMemcache())):
            self.assertTrue(memcached.set_('salt', '1111'))

            self.assertRaises(SaltInvocationError, memcached.set_,
                              'salt', '1111', time='0.1')

            self.assertRaises(SaltInvocationError, memcached.set_,
                              'salt', '1111', min_compress_len='0.1')

    # 'delete' function tests: 1

    def test_delete(self):
        '''
        Test if it delete a key from memcache server
        '''
        class MockMemcache(object):
            """
            Mock of memcache
            """
            def __init__(self):
                self.key = None
                self.time = None

            @staticmethod
            def get_stats():
                """
                Mock of stats method
                """
                return [('127.0.0.1:11211 (1)', {})]

            def delete(self, key, time):
                """
                Mock of delete method
                """
                self.key = key
                self.time = time
                return True

        with patch.object(memcached, '_connect',
                          MagicMock(return_value=MockMemcache())):
            self.assertTrue(memcached.delete('salt'))

            self.assertRaises(SaltInvocationError, memcached.delete,
                              'salt', '1111', time='0.1')

    # 'add' function tests: 1

    def test_add(self):
        '''
        Test if it add a key from memcache server
        '''
        class MockMemcache(object):
            """
            Mock of memcache
            """
            def __init__(self):
                self.key = None
                self.value = None
                self.time = None
                self.min_compress_len = None

            @staticmethod
            def get_stats():
                """
                Mock of stats method
                """
                return [('127.0.0.1:11211 (1)', {})]

            def add(self, key, value, time, min_compress_len):
                """
                Mock of add method
                """
                self.key = key
                self.value = value
                self.time = time
                self.min_compress_len = min_compress_len
                return True

        with patch.object(memcached, '_connect',
                          MagicMock(return_value=MockMemcache())):
            self.assertTrue(memcached.add('salt', '1111'))

            self.assertRaises(SaltInvocationError, memcached.add,
                              'salt', '1111', time='0.1')

            self.assertRaises(SaltInvocationError, memcached.add,
                              'salt', '1111', min_compress_len='0.1')

    # 'replace' function tests: 1

    def test_replace(self):
        '''
        Test if it replace a key from memcache server
        '''
        class MockMemcache(object):
            """
            Mock of memcache
            """
            def __init__(self):
                self.key = None
                self.value = None
                self.time = None
                self.min_compress_len = None

            @staticmethod
            def get_stats():
                """
                Mock of stats method
                """
                return [('127.0.0.1:11211 (1)', {})]

            def replace(self, key, value, time, min_compress_len):
                """
                Mock of replace method
                """
                self.key = key
                self.value = value
                self.time = time
                self.min_compress_len = min_compress_len
                return True

        with patch.object(memcached, '_connect',
                          MagicMock(return_value=MockMemcache())):
            self.assertTrue(memcached.replace('salt', '1111'))

            self.assertRaises(SaltInvocationError, memcached.replace,
                              'salt', '1111', time='0.1')

            self.assertRaises(SaltInvocationError, memcached.replace,
                              'salt', '1111', min_compress_len='0.1')

    # 'increment' function tests: 3

    def test_increment(self):
        '''
        Test if it increment the value of a key
        '''
        class MockMemcache(object):
            """
            Mock of memcache
            """
            def __init__(self):
                self.key = None

            @staticmethod
            def get_stats():
                """
                Mock of stats method
                """
                return [('127.0.0.1:11211 (1)', {})]

            def get(self, key):
                """
                Mock of get method
                """
                self.key = key
                return 1

            def incr(self, key, delta):
                """
                Mock of incr method
                """
                self.key = key
                if not isinstance(delta, integer_types):
                    raise SaltInvocationError('Delta value must be an integer')
                return key

        with patch.object(memcached, '_connect',
                          MagicMock(return_value=MockMemcache())):
            self.assertEqual(memcached.increment('salt'), 'salt')

            self.assertRaises(SaltInvocationError, memcached.increment,
                              'salt', delta='sa')

    def test_increment_exist(self):
        '''
        Test if it increment the value of a key
        '''
        class MockMemcache(object):
            """
            Mock of memcache
            """
            def __init__(self):
                self.key = None

            @staticmethod
            def get_stats():
                """
                Mock of stats method
                """
                return [('127.0.0.1:11211 (1)', {})]

            def get(self, key):
                """
                Mock of get method
                """
                self.key = key
                return key

        with patch.object(memcached, '_connect',
                          MagicMock(return_value=MockMemcache())):
            self.assertRaises(CommandExecutionError, memcached.increment,
                              'salt')

    def test_increment_none(self):
        '''
        Test if it increment the value of a key
        '''
        class MockMemcache(object):
            """
            Mock of memcache
            """
            def __init__(self):
                self.key = None

            @staticmethod
            def get_stats():
                """
                Mock of stats method
                """
                return [('127.0.0.1:11211 (1)', {})]

            def get(self, key):
                """
                Mock of get method
                """
                self.key = key
                return None

        with patch.object(memcached, '_connect',
                          MagicMock(return_value=MockMemcache())):
            self.assertRaises(CommandExecutionError, memcached.increment,
                              'salt')

    # 'decrement' function tests: 3

    def test_decrement(self):
        '''
        Test if it decrement the value of a key
        '''
        class MockMemcache(object):
            """
            Mock of memcache
            """
            def __init__(self):
                self.key = None

            @staticmethod
            def get_stats():
                """
                Mock of stats method
                """
                return [('127.0.0.1:11211 (1)', {})]

            def get(self, key):
                """
                Mock of get method
                """
                self.key = key
                return 1

            def decr(self, key, delta):
                """
                Mock of decr method
                """
                self.key = key
                if not isinstance(delta, integer_types):
                    raise SaltInvocationError('Delta value must be an integer')
                return key

        with patch.object(memcached, '_connect',
                          MagicMock(return_value=MockMemcache())):
            self.assertEqual(memcached.decrement('salt'), 'salt')

            self.assertRaises(SaltInvocationError, memcached.decrement,
                              'salt', delta='sa')

    def test_decrement_exist(self):
        '''
        Test if it decrement the value of a key
        '''
        class MockMemcache(object):
            """
            Mock of memcache
            """
            def __init__(self):
                self.key = None

            @staticmethod
            def get_stats():
                """
                Mock of stats method
                """
                return [('127.0.0.1:11211 (1)', {})]

            def get(self, key):
                """
                Mock of get method
                """
                self.key = key
                return key

        with patch.object(memcached, '_connect',
                          MagicMock(return_value=MockMemcache())):
            self.assertRaises(CommandExecutionError, memcached.decrement,
                              'salt')

    def test_decrement_none(self):
        '''
        Test if it decrement the value of a key
        '''
        class MockMemcache(object):
            """
            Mock of memcache
            """
            def __init__(self):
                self.key = None

            @staticmethod
            def get_stats():
                """
                Mock of stats method
                """
                return [('127.0.0.1:11211 (1)', {})]

            def get(self, key):
                """
                Mock of get method
                """
                self.key = key
                return None

        with patch.object(memcached, '_connect',
                          MagicMock(return_value=MockMemcache())):
            self.assertRaises(CommandExecutionError, memcached.decrement,
                              'salt')


if __name__ == '__main__':
    from integration import run_tests
    run_tests(MemcachedTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

from salttesting.helpers import ensure_in_syspath
from salt.exceptions import SaltInvocationError

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import boto_lc

boto_lc.__salt__ = {}
boto_lc.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class BotoLcTestCase(TestCase):
    '''
    Test cases for salt.states.boto_lc
    '''
    # 'present' function tests: 1

    def test_present(self):
        '''
        Test to ensure the launch configuration exists.
        '''
        name = 'mylc'
        image_id = 'ami-0b9c9f62'

        ret = {'name': name,
               'result': True,
               'changes': {},
               'comment': ''}

        self.assertRaises(SaltInvocationError, boto_lc.present, name,
                          image_id, user_data=True, cloud_init=True)

        mock = MagicMock(side_effect=[True, False])
        with patch.dict(boto_lc.__salt__,
                        {'boto_asg.launch_configuration_exists': mock}):
            comt = ('Launch configuration present.')
            ret.update({'comment': comt})
            self.assertDictEqual(boto_lc.present(name, image_id), ret)

            with patch.dict(boto_lc.__opts__, {'test': True}):
                comt = ('Launch configuration set to be created.')
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(boto_lc.present(name, image_id), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        '''
        Test to ensure the named launch configuration is deleted.
        '''
        name = 'mylc'

        ret = {'name': name,
               'result': True,
               'changes': {},
               'comment': ''}

        mock = MagicMock(side_effect=[False, True])
        with patch.dict(boto_lc.__salt__,
                        {'boto_asg.launch_configuration_exists': mock}):
            comt = ('Launch configuration does not exist.')
            ret.update({'comment': comt})
            self.assertDictEqual(boto_lc.absent(name), ret)

            with patch.dict(boto_lc.__opts__, {'test': True}):
                comt = ('Launch configuration set to be deleted.')
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(boto_lc.absent(name), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(BotoLcTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: `Anthony Shaw <anthonyshaw@apache.org>`

    tests.unit.cloud.clouds.dimensiondata_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase

# Import Salt Libs
import salt.cloud.libcloudfuncs as libcloud

# Import Salt Testing Libs
from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')


class LibcloudTestCase(TestCase):
    def test_node_state_libcloud_020(self):
        state = libcloud.node_state(2)
        self.assertEqual('TERMINATED', state)

    def test_node_state_libcloud_100(self):
        state = libcloud.node_state('terminated')
        self.assertEqual('TERMINATED', state)


if __name__ == '__main__':
    from unit import run_tests
    run_tests(LibcloudTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Top level package command wrapper, used to translate the os detected by grains
to the correct service manager

.. important::
    If you feel that Salt should be using this module to manage services on a
    minion, and it is using a different module (or gives an error similar to
    *'service.start' is not available*), see :ref:`here
    <module-provider-override>`.
'''

# Import Python libs
from __future__ import absolute_import
import logging

# Import salt libs
import salt.utils.systemd
import salt.utils.odict as odict

# Set up logging
log = logging.getLogger(__name__)

# Define the module's virtual name
__virtualname__ = 'service'


def __virtual__():
    '''
    Only work on systems which default to OpenRC
    '''
    if __grains__['os'] == 'Gentoo' and not salt.utils.systemd.booted(__context__):
        return __virtualname__
    return (False, 'The gentoo_service execution module cannot be loaded: '
            'only available on Gentoo/Open-RC systems.')


def _ret_code(cmd):
    log.debug('executing [{0}]'.format(cmd))
    sts = __salt__['cmd.retcode'](cmd, python_shell=False)
    return sts


def _list_services():
    return __salt__['cmd.run']('rc-update -v show').splitlines()


def _get_service_list(include_enabled=True, include_disabled=False):
    enabled_services = dict()
    disabled_services = set()
    lines = _list_services()
    for line in lines:
        if '|' not in line:
            continue
        service = [l.strip() for l in line.split('|')]
        # enabled service should have runlevels
        if service[1]:
            if include_enabled:
                enabled_services.update({service[0]: sorted(service[1].split())})
            continue
        # in any other case service is disabled
        if include_disabled:
            disabled_services.update({service[0]: []})
    return enabled_services, disabled_services


def _enable_delta(name, requested_runlevels):
    all_enabled = get_enabled()
    current_levels = set(all_enabled[name] if name in all_enabled else [])
    enabled_levels = requested_runlevels - current_levels
    disabled_levels = current_levels - requested_runlevels
    return enabled_levels, disabled_levels


def _disable_delta(name, requested_runlevels):
    all_enabled = get_enabled()
    current_levels = set(all_enabled[name] if name in all_enabled else [])
    return current_levels & requested_runlevels


def _service_cmd(*args):
    return '/etc/init.d/{0} {1}'.format(args[0], ' '.join(args[1:]))


def _enable_disable_cmd(name, command, runlevels=()):
    return 'rc-update {0} {1} {2}'.format(command, name, ' '.join(sorted(runlevels))).strip()


def get_enabled():
    '''
    Return a list of service that are enabled on boot

    CLI Example:

    .. code-block:: bash

        salt '*' service.get_enabled
    '''
    (enabled_services, disabled_services) = _get_service_list()
    return odict.OrderedDict(enabled_services)


def get_disabled():
    '''
    Return a set of services that are installed but disabled

    CLI Example:

    .. code-block:: bash

        salt '*' service.get_disabled
    '''
    (enabled_services, disabled_services) = _get_service_list(include_enabled=False,
                                                              include_disabled=True)
    return sorted(disabled_services)


def available(name):
    '''
    Returns ``True`` if the specified service is available, otherwise returns
    ``False``.

    CLI Example:

    .. code-block:: bash

        salt '*' service.available sshd
    '''
    (enabled_services, disabled_services) = _get_service_list(include_enabled=True,
                                                              include_disabled=True)
    return name in enabled_services or name in disabled_services


def missing(name):
    '''
    The inverse of service.available.
    Returns ``True`` if the specified service is not available, otherwise returns
    ``False``.

    CLI Example:

    .. code-block:: bash

        salt '*' service.missing sshd
    '''
    return not available(name)


def get_all():
    '''
    Return all available boot services

    CLI Example:

    .. code-block:: bash

        salt '*' service.get_all
    '''
    (enabled_services, disabled_services) = _get_service_list(include_enabled=True,
                                                              include_disabled=True)
    enabled_services.update(dict([(s, []) for s in disabled_services]))
    return odict.OrderedDict(enabled_services)


def start(name):
    '''
    Start the specified service

    CLI Example:

    .. code-block:: bash

        salt '*' service.start <service name>
    '''
    cmd = _service_cmd(name, 'start')
    return not _ret_code(cmd)


def stop(name):
    '''
    Stop the specified service

    CLI Example:

    .. code-block:: bash

        salt '*' service.stop <service name>
    '''
    cmd = _service_cmd(name, 'stop')
    return not _ret_code(cmd)


def restart(name):
    '''
    Restart the named service

    CLI Example:

    .. code-block:: bash

        salt '*' service.restart <service name>
    '''
    cmd = _service_cmd(name, 'restart')
    return not _ret_code(cmd)


def reload_(name):
    '''
    Reload the named service

    CLI Example:

    .. code-block:: bash

        salt '*' service.reload <service name>
    '''
    cmd = _service_cmd(name, 'reload')
    return not _ret_code(cmd)


def zap(name):
    '''
    Resets service state

    CLI Example:

    .. code-block:: bash

        salt '*' service.zap <service name>
    '''
    cmd = _service_cmd(name, 'zap')
    return not _ret_code(cmd)


def status(name, sig=None):
    '''
    Return the status for a service, returns the PID or an empty string if the
    service is running or not, pass a signature to use to find the service via
    ps

    CLI Example:

    .. code-block:: bash

        salt '*' service.status <service name> [service signature]
    '''
    if sig:
        return bool(__salt__['status.pid'](sig))
    cmd = _service_cmd(name, 'status')
    return not _ret_code(cmd)


def enable(name, **kwargs):
    '''
    Enable the named service to start at boot

    CLI Example:

    .. code-block:: bash

        salt '*' service.enable <service name> <runlevels=single-runlevel>
        salt '*' service.enable <service name> <runlevels=[runlevel1,runlevel2]>
    '''
    if 'runlevels' in kwargs:
        requested_levels = set(kwargs['runlevels'] if isinstance(kwargs['runlevels'],
                                                                 list) else [kwargs['runlevels']])
        enabled_levels, disabled_levels = _enable_delta(name, requested_levels)
        commands = []
        if disabled_levels:
            commands.append(_enable_disable_cmd(name, 'delete', disabled_levels))
        if enabled_levels:
            commands.append(_enable_disable_cmd(name, 'add', enabled_levels))
        if not commands:
            return True
    else:
        commands = [_enable_disable_cmd(name, 'add')]
    for cmd in commands:
        if _ret_code(cmd):
            return False
    return True


def disable(name, **kwargs):
    '''
    Disable the named service to start at boot

    CLI Example:

    .. code-block:: bash

        salt '*' service.disable <service name> <runlevels=single-runlevel>
        salt '*' service.disable <service name> <runlevels=[runlevel1,runlevel2]>
    '''
    levels = []
    if 'runlevels' in kwargs:
        requested_levels = set(kwargs['runlevels'] if isinstance(kwargs['runlevels'],
                                                                 list) else [kwargs['runlevels']])
        levels = _disable_delta(name, requested_levels)
        if not levels:
            return True
    cmd = _enable_disable_cmd(name, 'delete', levels)
    return not _ret_code(cmd)


def enabled(name, **kwargs):
    '''
    Return True if the named service is enabled, false otherwise

    CLI Example:

    .. code-block:: bash

        salt '*' service.enabled <service name> <runlevels=single-runlevel>
        salt '*' service.enabled <service name> <runlevels=[runlevel1,runlevel2]>
    '''
    enabled_services = get_enabled()
    if name not in enabled_services:
        return False
    if 'runlevels' not in kwargs:
        return True
    requested_levels = set(kwargs['runlevels'] if isinstance(kwargs['runlevels'],
                                                             list) else [kwargs['runlevels']])
    return len(requested_levels - set(enabled_services[name])) == 0


def disabled(name):
    '''
    Return True if the named service is enabled, false otherwise

    CLI Example:

    .. code-block:: bash

        salt '*' service.disabled <service name> <runlevels=[runlevel]>
    '''
    return name in get_disabled()



# -*- coding: utf-8 -*-
'''
Module for returning various status data about a minion.
These data can be useful for compiling into stats later,
or for problem solving if your minion is having problems.

.. versionadded:: 0.12.0

:depends:   - pythoncom
            - wmi
'''

# Import Python Libs
from __future__ import absolute_import
import os
import ctypes
import sys
import time
import datetime
import logging

log = logging.getLogger(__name__)

# Import Salt Libs
import salt.utils
import salt.ext.six as six
import salt.utils.event
from salt._compat import subprocess
from salt.utils.network import host_to_ips as _host_to_ips
# pylint: disable=W0611
from salt.modules.status import ping_master, time_
import copy
# pylint: enable=W0611
from salt.utils import namespaced_function as _namespaced_function

# Import 3rd Party Libs
if salt.utils.is_windows():
    import wmi
    import salt.utils.winapi
    HAS_WMI = True
else:
    HAS_WMI = False

__opts__ = {}
__virtualname__ = 'status'


def __virtual__():
    '''
    Only works on Windows systems with WMI and WinAPI
    '''
    if not salt.utils.is_windows():
        return False, 'win_status.py: Requires Windows'

    if not HAS_WMI:
        return False, 'win_status.py: Requires WMI and WinAPI'

    # Namespace modules from `status.py`
    global ping_master, time_
    ping_master = _namespaced_function(ping_master, globals())
    time_ = _namespaced_function(time_, globals())

    return __virtualname__

__func_alias__ = {
    'time_': 'time'
}


def cpuload():
    '''
    .. versionadded:: 2015.8.0

    Return the processor load as a percentage

    CLI Example:

    .. code-block:: bash

       salt '*' status.cpuload
    '''

    # Pull in the information from WMIC
    cmd = ['wmic', 'cpu', 'get', 'loadpercentage', '/value']
    return int(__salt__['cmd.run'](cmd).split('=')[1])


def diskusage(human_readable=False, path=None):
    '''
    .. versionadded:: 2015.8.0

    Return the disk usage for this minion

    human_readable : False
        If ``True``, usage will be in KB/MB/GB etc.

    CLI Example:

    .. code-block:: bash

        salt '*' status.diskusage path=c:/salt
    '''
    if not path:
        path = 'c:/'

    # Credit for the source and ideas for this function:
    # http://code.activestate.com/recipes/577972-disk-usage/?in=user-4178764
    _, total, free = \
        ctypes.c_ulonglong(), ctypes.c_ulonglong(), ctypes.c_longlong()
    if sys.version_info >= (3, ) or isinstance(path, six.text_type):
        fun = ctypes.windll.kernel32.GetDiskFreeSpaceExW
    else:
        fun = ctypes.windll.kernel32.GetDiskFreeSpaceExA
    ret = fun(path, ctypes.byref(_), ctypes.byref(total), ctypes.byref(free))
    if ret == 0:
        raise ctypes.WinError()
    used = total.value - free.value

    total_val = total.value
    used_val = used
    free_val = free.value

    if human_readable:
        total_val = _byte_calc(total_val)
        used_val = _byte_calc(used_val)
        free_val = _byte_calc(free_val)

    return {'total': total_val, 'used': used_val, 'free': free_val}


def procs(count=False):
    '''
    Return the process data

    count : False
        If ``True``, this function will simply return the number of processes.

        .. versionadded:: 2015.8.0

    CLI Example:

    .. code-block:: bash

        salt '*' status.procs
        salt '*' status.procs count
    '''
    with salt.utils.winapi.Com():
        wmi_obj = wmi.WMI()
        processes = wmi_obj.win32_process()

    #this short circuit's the function to get a short simple proc count.
    if count:
        return len(processes)

    #a propper run of the function, creating a nonsensically long out put.
    process_info = {}
    for proc in processes:
        process_info[proc.ProcessId] = _get_process_info(proc)

    return process_info


def saltmem(human_readable=False):
    '''
    .. versionadded:: 2015.8.0

    Returns the amount of memory that salt is using

    human_readable : False
        return the value in a nicely formatted number

    CLI Example:

    .. code-block:: bash

        salt '*' status.saltmem
        salt '*' status.saltmem human_readable=True
    '''
    with salt.utils.winapi.Com():
        wmi_obj = wmi.WMI()
        result = wmi_obj.query(
            'SELECT WorkingSet FROM Win32_PerfRawData_PerfProc_Process '
            'WHERE IDProcess={0}'.format(os.getpid())
        )
        mem = int(result[0].wmi_property('WorkingSet').value)
        if human_readable:
            return _byte_calc(mem)
        return mem


def uptime(human_readable=False):
    '''
    .. versionadded:: 2015.8.0

    Return the system uptime for this machine in seconds

    human_readable : False
        If ``True``, then return uptime in years, days, and seconds.

    CLI Example:

    .. code-block:: bash

       salt '*' status.uptime
       salt '*' status.uptime human_readable=True
    '''

    # Open up a subprocess to get information from WMIC
    cmd = ['wmic', 'os', 'get', 'lastbootuptime', '/value']
    startup_time = __salt__['cmd.run'](cmd).split('=')[1][:14]

    # Convert to time struct
    startup_time = time.strptime(startup_time, '%Y%m%d%H%M%S')
    # Convert to datetime object
    startup_time = datetime.datetime(*startup_time[:6])

    # Subtract startup time from current time to get the uptime of the system
    uptime = datetime.datetime.now() - startup_time

    return str(uptime) if human_readable else uptime.total_seconds()


def _get_process_info(proc):
    '''
    Return  process information
    '''
    cmd = salt.utils.to_str(proc.CommandLine or '')
    name = salt.utils.to_str(proc.Name)
    info = dict(
        cmd=cmd,
        name=name,
        **_get_process_owner(proc)
    )
    return info


def _get_process_owner(process):
    owner = {}
    domain, error_code, user = None, None, None
    try:
        domain, error_code, user = process.GetOwner()
        owner['user'] = salt.utils.to_str(user)
        owner['user_domain'] = salt.utils.to_str(domain)
    except Exception as exc:
        pass
    if not error_code and all((user, domain)):
        owner['user'] = salt.utils.to_str(user)
        owner['user_domain'] = salt.utils.to_str(domain)
    elif process.ProcessId in [0, 4] and error_code == 2:
        # Access Denied for System Idle Process and System
        owner['user'] = 'SYSTEM'
        owner['user_domain'] = 'NT AUTHORITY'
    else:
        log.warning('Error getting owner of process; PID=\'{0}\'; Error: {1}'
                    .format(process.ProcessId, error_code))
    return owner


def _byte_calc(val):
    if val < 1024:
        tstr = str(val)+'B'
    elif val < 1038336:
        tstr = str(val/1024)+'KB'
    elif val < 1073741824:
        tstr = str(val/1038336)+'MB'
    elif val < 1099511627776:
        tstr = str(val/1073741824)+'GB'
    else:
        tstr = str(val/1099511627776)+'TB'
    return tstr


def master(master=None, connected=True):
    '''
    .. versionadded:: 2015.5.0

    Fire an event if the minion gets disconnected from its master. This
    function is meant to be run via a scheduled job from the minion. If
    master_ip is an FQDN/Hostname, is must be resolvable to a valid IPv4
    address.

    CLI Example:

    .. code-block:: bash

        salt '*' status.master
    '''

    def _win_remotes_on(port):
        '''
        Windows specific helper function.
        Returns set of ipv4 host addresses of remote established connections
        on local or remote tcp port.

        Parses output of shell 'netstat' to get connections

        PS C:> netstat -n -p TCP

        Active Connections

          Proto  Local Address          Foreign Address        State
          TCP    10.1.1.26:3389         10.1.1.1:4505          ESTABLISHED
          TCP    10.1.1.26:56862        10.1.1.10:49155        TIME_WAIT
          TCP    10.1.1.26:56868        169.254.169.254:80     CLOSE_WAIT
          TCP    127.0.0.1:49197        127.0.0.1:49198        ESTABLISHED
          TCP    127.0.0.1:49198        127.0.0.1:49197        ESTABLISHED
        '''
        remotes = set()
        try:
            data = subprocess.check_output(['netstat', '-n', '-p', 'TCP'])  # pylint: disable=minimum-python-version
        except subprocess.CalledProcessError:
            log.error('Failed netstat')
            raise

        lines = salt.utils.to_str(data).split('\n')
        for line in lines:
            if 'ESTABLISHED' not in line:
                continue
            chunks = line.split()
            remote_host, remote_port = chunks[2].rsplit(':', 1)
            if int(remote_port) != port:
                continue
            remotes.add(remote_host)
        return remotes

    # the default publishing port
    port = 4505
    master_ips = None

    if __salt__['config.get']('publish_port') != '':
        port = int(__salt__['config.get']('publish_port'))

    # Check if we have FQDN/hostname defined as master
    # address and try resolving it first. _remote_port_tcp
    # only works with IP-addresses.
    if master is not None:
        master_ips = _host_to_ips(master)

    master_connection_status = False
    if master_ips:
        ips = _win_remotes_on(port)
        for master_ip in master_ips:
            if master_ip in ips:
                master_connection_status = True
                break

    if master_connection_status is not connected:
        event = salt.utils.event.get_event('minion', opts=__opts__, listen=False)
        if master_connection_status:
            event.fire_event({'master': master}, salt.minion.master_event(type='connected'))
        else:
            event.fire_event({'master': master}, salt.minion.master_event(type='disconnected'))

    return master_connection_status



# -*- coding: utf-8 -*-
'''
NAPALM Users
============

Manages the configuration of the users on network devices.

:codeauthor: Mircea Ulinic <mircea@cloudflare.com>
:maturity:   new
:depends:    napalm
:platform:   unix

Dependencies
------------
- :mod:`NAPALM proxy minion <salt.proxy.napalm>`

.. seealso::
    :mod:`Users management state <salt.states.netusers>`

.. versionadded:: 2016.11.0
'''

from __future__ import absolute_import

import logging
log = logging.getLogger(__file__)


try:
    # will try to import NAPALM
    # https://github.com/napalm-automation/napalm
    # pylint: disable=W0611
    from napalm_base import get_network_driver
    # pylint: enable=W0611
    HAS_NAPALM = True
except ImportError:
    HAS_NAPALM = False

# ----------------------------------------------------------------------------------------------------------------------
# module properties
# ----------------------------------------------------------------------------------------------------------------------

__virtualname__ = 'users'
__proxyenabled__ = ['napalm']
# uses NAPALM-based proxy to interact with network devices

# ----------------------------------------------------------------------------------------------------------------------
# property functions
# ----------------------------------------------------------------------------------------------------------------------


def __virtual__():

    '''
    NAPALM library must be installed for this module to work.
    Also, the key proxymodule must be set in the __opts___ dictionary.
    '''

    if HAS_NAPALM and 'proxy' in __opts__:
        return __virtualname__
    else:
        return (False, 'The module napalm_users cannot be loaded: \
                NAPALM or proxy could not be loaded.')

# ----------------------------------------------------------------------------------------------------------------------
# helper functions -- will not be exported
# ----------------------------------------------------------------------------------------------------------------------

# ----------------------------------------------------------------------------------------------------------------------
# callable functions
# ----------------------------------------------------------------------------------------------------------------------


def config():

    '''
    Returns the configuration of the users on the device

    CLI Example:

    .. code-block:: bash

        salt '*' users.config

    Output example:

    .. code-block:: python

        {
            'mircea': {
                'level': 15,
                'password': '$1$0P70xKPa$4jt5/10cBTckk6I/w/',
                'sshkeys': [
                    'ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQC4pFn+shPwTb2yELO4L7NtQrKOJXNeCl1je\
                    l9STXVaGnRAnuc2PXl35vnWmcUq6YbUEcgUTRzzXfmelJKuVJTJIlMXii7h2xkbQp0YZIEs4P\
                    8ipwnRBAxFfk/ZcDsN3mjep4/yjN56ejk345jhk345jk345jk341p3A/9LIL7l6YewLBCwJj6\
                    D+fWSJ0/YW+7oH17Fk2HH+tw0L5PcWLHkwA4t60iXn16qDbIk/ze6jv2hDGdCdz7oYQeCE55C\
                    CHOHMJWYfN3jcL4s0qv8/u6Ka1FVkV7iMmro7ChThoV/5snI4Ljf2wKqgHH7TfNaCfpU0WvHA\
                    nTs8zhOrGScSrtb mircea@master-roshi'
                ]
            }
        }
    '''

    return __proxy__['napalm.call'](
        'get_users',
        **{
        }
    )


def set_users(users, test=False, commit=True):

    '''
    Configures users on network devices.

    :param users: Dictionary formatted as the output of the function config()
    :param test: Dry run? If set as True, will apply the config, discard and return the changes. Default: False
    and will commit the changes on the device.
    :param commit: Commit? (default: True) Sometimes it is not needed to commit the config immediately
        after loading the changes. E.g.: a state loads a couple of parts (add / remove / update)
        and would not be optimal to commit after each operation.
        Also, from the CLI when the user needs to apply the similar changes before committing,
        can specify commit=False and will not discard the config.
    :raise MergeConfigException: If there is an error on the configuration sent.
    :return a dictionary having the following keys:
        * result (bool): if the config was applied successfully. It is `False` only in case of failure. In case
        there are no changes to be applied and successfully performs all operations it is still `True` and so will be
        the `already_configured` flag (example below)
        * comment (str): a message for the user
        * already_configured (bool): flag to check if there were no changes applied
        * diff (str): returns the config changes applied

    CLI Example:

    .. code-block:: bash

        salt '*' users.set_users "{'mircea': {}}"
    '''

    return __salt__['net.load_template']('set_users',
                                         users=users,
                                         test=test,
                                         commit=commit)


def delete_users(users, test=False, commit=True):

    '''
    Removes users from the configuration of network devices.

    :param users: Dictionary formatted as the output of the function config()
    :param test: Dry run? If set as True, will apply the config, discard and return the changes. Default: False
    and will commit the changes on the device.
    :param commit: Commit? (default: True) Sometimes it is not needed to commit the config immediately
        after loading the changes. E.g.: a state loads a couple of parts (add / remove / update)
        and would not be optimal to commit after each operation.
        Also, from the CLI when the user needs to apply the similar changes before committing,
        can specify commit=False and will not discard the config.
    :raise MergeConfigException: If there is an error on the configuration sent.
    :return a dictionary having the following keys:
        * result (bool): if the config was applied successfully. It is `False` only in case of failure. In case
        there are no changes to be applied and successfully performs all operations it is still `True` and so will be
        the `already_configured` flag (example below)
        * comment (str): a message for the user
        * already_configured (bool): flag to check if there were no changes applied
        * diff (str): returns the config changes applied

    CLI Example:

    .. code-block:: bash

        salt '*' users.delete_users "{'mircea': {}}"
    '''

    return __salt__['net.load_template']('delete_users',
                                         users=users,
                                         test=test,
                                         commit=commit)



# -*- coding: utf-8 -*-
'''
Management of InfluxDB users
============================

(compatible with InfluxDB version 0.9+)
'''


def __virtual__():
    '''
    Only load if the influxdb module is available
    '''
    if 'influxdb.db_exists' in __salt__:
        return 'influxdb_user'
    return False


def _changes(name, admin, **client_args):
    '''
    Get necessary changes to given user account
    '''

    existing_user = __salt__['influxdb.user_info'](name, **client_args)
    changes = {}

    if existing_user['admin'] != admin:
        changes['admin'] = admin

    return changes


def present(name,
            password,
            admin=False,
            **client_args):
    '''
    Ensure that given user is present.

    name
        Name of the user to manage

    password
        Password of the user

    admin : False
        Whether the user should have cluster administration
        privileges or not.
    '''
    ret = {'name': name,
           'changes': {},
           'result': True,
           'comment': 'User {0} is present and up to date'.format(name)}

    if not __salt__['influxdb.user_exists'](name, **client_args):
        if __opts__['test']:
            ret['result'] = None
            ret['comment'] = 'User {0} will be created'.format(name)
            return ret
        else:
            if __salt__['influxdb.create_user'](
                    name, password, admin=admin, **client_args):
                ret['comment'] = 'Created user {0}'.format(name)
                ret['changes'][name] = 'created'
                return ret
            else:
                ret['comment'] = 'Failed to create user {0}'.format(name)
                ret['result'] = False
                return ret
    else:
        changes = _changes(name, admin, **client_args)
        if changes:
            if __opts__['test']:
                ret['result'] = None
                ret['comment'] = ('The following user attributes are set to '
                                  'be changed:\n')
                for k, v in changes.items():
                    ret['comment'] += u'{0}: {1}\n'.format(k, v)
                return ret
            else:
                pre = __salt__['influxdb.user_info'](name, **client_args)
                for k, v in changes.items():
                    if k == 'admin':
                        if v:
                            __salt__['influxdb.grant_admin_privileges'](name, **client_args)
                            continue
                        else:
                            __salt__['influxdb.revoke_admin_privileges'](name, **client_args)
                            continue

                post = __salt__['influxdb.user_info'](name, **client_args)
                for k in post:
                    if post[k] != pre[k]:
                        ret['changes'][k] = post[k]
                if ret['changes']:
                    ret['comment'] = 'Updated user {0}'.format(name)
    return ret


def absent(name, **client_args):
    '''
    Ensure that given user is absent.

    name
        The name of the user to manage
    '''
    ret = {'name': name,
           'changes': {},
           'result': True,
           'comment': 'User {0} is not present'.format(name)}

    if __salt__['influxdb.user_exists'](name, **client_args):
        if __opts__['test']:
            ret['result'] = None
            ret['comment'] = 'User {0} will be removed'.format(name)
            return ret
        else:
            if __salt__['influxdb.remove_user'](name, **client_args):
                ret['comment'] = 'Removed user {0}'.format(name)
                ret['changes'][name] = 'removed'
                return ret
            else:
                ret['comment'] = 'Failed to remove user {0}'.format(name)
                ret['result'] = False
                return ret
    return ret



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import
import copy

# Import Salt Testing libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

ensure_in_syspath('../../')

# Import Salt libs
from salt.exceptions import SaltException
from salt.modules import grains as grainsmod
from salt.utils import dictupdate

# Import 3rd-party libs
from salt.utils.odict import OrderedDict

grainsmod.__opts__ = {
  'conf_file': '/tmp/__salt_test_grains',
  'cachedir':  '/tmp/__salt_test_grains_cache_dir'
}

grainsmod.__salt__ = {}


@patch.dict(grainsmod.__salt__, {'saltutil.refresh_grains': MagicMock()})
@skipIf(NO_MOCK, NO_MOCK_REASON)
class GrainsModuleTestCase(TestCase):

    def test_filter_by(self):
        grainsmod.__grains__ = {
            'os_family': 'MockedOS',
            '1': '1',
            '2': '2',
            'roles': ['A', 'B'],
        }

        dict1 = {'A': 'B', 'C': {'D': {'E': 'F', 'G': 'H'}}}
        dict2 = {
            'default': {
                'A': 'B',
                'C': {
                    'D': 'E'
                },
            },
            '1': {
                'A': 'X',
            },
            '2': {
                'C': {
                    'D': 'H',
                },
            },
            'MockedOS': {
                'A': 'Z',
            },
        }

        mdict1 = {'D': {'E': 'I'}, 'J': 'K'}
        mdict2 = {'A': 'Z'}
        mdict3 = {'C': {'D': 'J'}}

        # test None result with non existent grain and no default
        res = grainsmod.filter_by(dict1, grain='xxx')
        self.assertIs(res, None)

        # test None result with os_family grain and no matching result
        res = grainsmod.filter_by(dict1)
        self.assertIs(res, None)

        # test with non existent grain, and a given default key
        res = grainsmod.filter_by(dict1, grain='xxx', default='C')
        self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})

        # add a merge dictionary, F disappears
        res = grainsmod.filter_by(dict1, grain='xxx', merge=mdict1, default='C')
        self.assertEqual(res, {'D': {'E': 'I', 'G': 'H'}, 'J': 'K'})
        # dict1 was altered, reestablish
        dict1 = {'A': 'B', 'C': {'D': {'E': 'F', 'G': 'H'}}}

        # default is not present in dict1, check we only have merge in result
        res = grainsmod.filter_by(dict1, grain='xxx', merge=mdict1, default='Z')
        self.assertEqual(res, mdict1)

        # default is not present in dict1, and no merge, should get None
        res = grainsmod.filter_by(dict1, grain='xxx', default='Z')
        self.assertIs(res, None)

        #test giving a list as merge argument raise exception
        self.assertRaises(
            SaltException,
            grainsmod.filter_by,
            dict1,
            'xxx',
            ['foo'],
            'C'
        )

        #Now, re-test with an existing grain (os_family), but with no match.
        res = grainsmod.filter_by(dict1)
        self.assertIs(res, None)
        res = grainsmod.filter_by(dict1, default='C')
        self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})
        res = grainsmod.filter_by(dict1, merge=mdict1, default='C')
        self.assertEqual(res, {'D': {'E': 'I', 'G': 'H'}, 'J': 'K'})
        # dict1 was altered, reestablish
        dict1 = {'A': 'B', 'C': {'D': {'E': 'F', 'G': 'H'}}}
        res = grainsmod.filter_by(dict1, merge=mdict1, default='Z')
        self.assertEqual(res, mdict1)
        res = grainsmod.filter_by(dict1, default='Z')
        self.assertIs(res, None)
        # this one is in fact a traceback in updatedict, merging a string with a dictionary
        self.assertRaises(
            TypeError,
            grainsmod.filter_by,
            dict1,
            merge=mdict1,
            default='A'
        )

        #Now, re-test with a matching grain.
        dict1 = {'A': 'B', 'MockedOS': {'D': {'E': 'F', 'G': 'H'}}}
        res = grainsmod.filter_by(dict1)
        self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})
        res = grainsmod.filter_by(dict1, default='A')
        self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})
        res = grainsmod.filter_by(dict1, merge=mdict1, default='A')
        self.assertEqual(res, {'D': {'E': 'I', 'G': 'H'}, 'J': 'K'})
        # dict1 was altered, reestablish
        dict1 = {'A': 'B', 'MockedOS': {'D': {'E': 'F', 'G': 'H'}}}
        res = grainsmod.filter_by(dict1, merge=mdict1, default='Z')
        self.assertEqual(res, {'D': {'E': 'I', 'G': 'H'}, 'J': 'K'})
        # dict1 was altered, reestablish
        dict1 = {'A': 'B', 'MockedOS': {'D': {'E': 'F', 'G': 'H'}}}
        res = grainsmod.filter_by(dict1, default='Z')
        self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})

        # Test when grain value is a list
        dict1 = {'A': 'B', 'C': {'D': {'E': 'F', 'G': 'H'}}}
        res = grainsmod.filter_by(dict1, grain='roles', default='C')
        self.assertEqual(res, 'B')
        # Test default when grain value is a list
        dict1 = {'Z': 'B', 'C': {'D': {'E': 'F', 'G': 'H'}}}
        res = grainsmod.filter_by(dict1, grain='roles', default='C')
        self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})

        # Test with wildcard pattern in the lookup_dict keys
        dict1 = {'*OS': 'B', 'C': {'D': {'E': 'F', 'G': 'H'}}}
        res = grainsmod.filter_by(dict1)
        self.assertEqual(res, 'B')
        # Test with non-strings in lookup_dict keys
        # Issue #38094
        dict1 = {1: 2, 3: {4: 5}, '*OS': 'B'}
        res = grainsmod.filter_by(dict1)
        self.assertEqual(res, 'B')
        # Test with sequence pattern with roles
        dict1 = {'Z': 'B', '[BC]': {'D': {'E': 'F', 'G': 'H'}}}
        res = grainsmod.filter_by(dict1, grain='roles', default='Z')
        self.assertEqual(res, {'D': {'E': 'F', 'G': 'H'}})

        # Base tests
        # NOTE: these may fail to detect errors if dictupdate.update() is broken
        # but then the unit test for dictupdate.update() should fail and expose
        # that.  The purpose of these tests is it validate the logic of how
        # in filter_by() processes its arguments.

        # Test with just the base
        res = grainsmod.filter_by(dict2, grain='xxx', default='xxx', base='default')
        self.assertEqual(res, dict2['default'])

        # Test the base with the OS grain look-up
        res = grainsmod.filter_by(dict2, default='xxx', base='default')
        self.assertEqual(
            res,
            dictupdate.update(copy.deepcopy(dict2['default']), dict2['MockedOS'])
        )

        # Test the base with default
        res = grainsmod.filter_by(dict2, grain='xxx', base='default')
        self.assertEqual(res, dict2['default'])

        res = grainsmod.filter_by(dict2, grain='1', base='default')
        self.assertEqual(
            res,
            dictupdate.update(copy.deepcopy(dict2['default']), dict2['1'])
        )

        res = grainsmod.filter_by(dict2, base='default', merge=mdict2)
        self.assertEqual(
            res,
            dictupdate.update(
                dictupdate.update(
                    copy.deepcopy(dict2['default']),
                    dict2['MockedOS']),
                mdict2
            )
        )

        res = grainsmod.filter_by(dict2, base='default', merge=mdict3)
        self.assertEqual(
            res,
            dictupdate.update(
                dictupdate.update(
                    copy.deepcopy(dict2['default']),
                    dict2['MockedOS']),
                mdict3
            )
        )

    def test_append_not_a_list(self):
        # Failing append to an existing string, without convert
        grainsmod.__grains__ = {'b': 'bval'}
        res = grainsmod.append('b', 'd')
        self.assertEqual(res, 'The key b is not a valid list')
        self.assertEqual(grainsmod.__grains__, {'b': 'bval'})

        # Failing append to an existing dict
        grainsmod.__grains__ = {'b': {'b1': 'bval1'}}
        res = grainsmod.append('b', 'd')
        self.assertEqual(res, 'The key b is not a valid list')
        self.assertEqual(grainsmod.__grains__, {'b': {'b1': 'bval1'}})

    def test_append_already_in_list(self):
        # Append an existing value
        grainsmod.__grains__ = {'a_list': ['a', 'b', 'c'], 'b': 'bval'}
        res = grainsmod.append('a_list', 'b')
        self.assertEqual(res, 'The val b was already in the list a_list')
        self.assertEqual(grainsmod.__grains__, {'a_list': ['a', 'b', 'c'], 'b': 'bval'})

    def test_append_ok(self):
        # Append to an existing list
        grainsmod.__grains__ = {'a_list': ['a', 'b', 'c'], 'b': 'bval'}
        res = grainsmod.append('a_list', 'd')
        self.assertEqual(res, {'a_list': ['a', 'b', 'c', 'd']})
        self.assertEqual(grainsmod.__grains__, {'a_list': ['a', 'b', 'c', 'd'], 'b': 'bval'})

        # Append to an non existing list
        grainsmod.__grains__ = {'b': 'bval'}
        res = grainsmod.append('a_list', 'd')
        self.assertEqual(res, {'a_list': ['d']})
        self.assertEqual(grainsmod.__grains__, {'a_list': ['d'], 'b': 'bval'})

        # Append to an existing string, with convert
        grainsmod.__grains__ = {'b': 'bval'}
        res = grainsmod.append('b', 'd', convert=True)
        self.assertEqual(res, {'b': ['bval', 'd']})
        self.assertEqual(grainsmod.__grains__, {'b': ['bval', 'd']})

        # Append to an existing dict, with convert
        grainsmod.__grains__ = {'b': {'b1': 'bval1'}}
        res = grainsmod.append('b', 'd', convert=True)
        self.assertEqual(res, {'b': [{'b1': 'bval1'}, 'd']})
        self.assertEqual(grainsmod.__grains__, {'b': [{'b1': 'bval1'}, 'd']})

    def test_append_nested_not_a_list(self):
        # Failing append to an existing string, without convert
        grainsmod.__grains__ = {'a': {'b': 'bval'}}
        res = grainsmod.append('a:b', 'd')
        self.assertEqual(res, 'The key a:b is not a valid list')
        self.assertEqual(grainsmod.__grains__, {'a': {'b': 'bval'}})

        # Failing append to an existing dict
        grainsmod.__grains__ = {'a': {'b': {'b1': 'bval1'}}}
        res = grainsmod.append('a:b', 'd')
        self.assertEqual(res, 'The key a:b is not a valid list')
        self.assertEqual(grainsmod.__grains__, {'a': {'b': {'b1': 'bval1'}}})

    def test_append_nested_already_in_list(self):
        # Append an existing value
        grainsmod.__grains__ = {'a': {'a_list': ['a', 'b', 'c'], 'b': 'bval'}}
        res = grainsmod.append('a:a_list', 'b')
        self.assertEqual(res, 'The val b was already in the list a:a_list')
        self.assertEqual(grainsmod.__grains__, {'a': {'a_list': ['a', 'b', 'c'], 'b': 'bval'}})

    def test_append_nested_ok(self):
        # Append to an existing list
        grainsmod.__grains__ = {'a': {'a_list': ['a', 'b', 'c'], 'b': 'bval'}}
        res = grainsmod.append('a:a_list', 'd')
        self.assertEqual(res, {'a': {'a_list': ['a', 'b', 'c', 'd'], 'b': 'bval'}})
        self.assertEqual(grainsmod.__grains__, {'a': {'a_list': ['a', 'b', 'c', 'd'], 'b': 'bval'}})

        # Append to an non existing list
        grainsmod.__grains__ = {'a': {'b': 'bval'}}
        res = grainsmod.append('a:a_list', 'd')
        self.assertEqual(res, {'a': {'a_list': ['d'], 'b': 'bval'}})
        self.assertEqual(grainsmod.__grains__, {'a': {'a_list': ['d'], 'b': 'bval'}})

        # Append to an existing string, with convert
        grainsmod.__grains__ = {'a': {'b': 'bval'}}
        res = grainsmod.append('a:b', 'd', convert=True)
        self.assertEqual(res, {'a': {'b': ['bval', 'd']}})
        self.assertEqual(grainsmod.__grains__, {'a': {'b': ['bval', 'd']}})

        # Append to an existing dict, with convert
        grainsmod.__grains__ = {'a': {'b': {'b1': 'bval1'}}}
        res = grainsmod.append('a:b', 'd', convert=True)
        self.assertEqual(res, {'a': {'b': [{'b1': 'bval1'}, 'd']}})
        self.assertEqual(grainsmod.__grains__, {'a': {'b': [{'b1': 'bval1'}, 'd']}})

    def test_append_to_an_element_of_a_list(self):
        # Append to an element in a list
        # It currently fails silently
        grainsmod.__grains__ = {'a': ['b', 'c']}
        res = grainsmod.append('a:b', 'd')
        self.assertEqual(res, {'a': ['b', 'c']})
        self.assertEqual(grainsmod.__grains__, {'a': ['b', 'c']})

    def test_set_value_already_set(self):
        # Set a grain to the same simple value
        grainsmod.__grains__ = {'a': 12, 'c': 8}
        res = grainsmod.set('a', 12)
        self.assertTrue(res['result'])
        self.assertEqual(res['comment'], 'Grain is already set')
        self.assertEqual(grainsmod.__grains__, {'a': 12, 'c': 8})

        # Set a grain to the same complex value
        grainsmod.__grains__ = {'a': ['item', 12], 'c': 8}
        res = grainsmod.set('a', ['item', 12])
        self.assertTrue(res['result'])
        self.assertEqual(res['comment'], 'Grain is already set')
        self.assertEqual(grainsmod.__grains__, {'a': ['item', 12], 'c': 8})

        # Set a key to the same simple value in a nested grain
        grainsmod.__grains__ = {'a': 'aval', 'b': {'nested': 'val'}, 'c': 8}
        res = grainsmod.set('b,nested', 'val', delimiter=',')
        self.assertTrue(res['result'])
        self.assertEqual(res['comment'], 'Grain is already set')
        self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                'b': {'nested': 'val'},
                                                'c': 8})

    def test_set_fail_replacing_existing_complex_key(self):
        # Fails to set a complex value without 'force'
        grainsmod.__grains__ = {'a': 'aval', 'c': 8}
        res = grainsmod.set('a', ['item', 12])
        self.assertFalse(res['result'])
        self.assertEqual(res['comment'], 'The key \'a\' exists and the given value is a '
                            + 'dict or a list. Use \'force=True\' to overwrite.')
        self.assertEqual(grainsmod.__grains__, {'a': 'aval', 'c': 8})

        # Fails to overwrite a complex value without 'force'
        grainsmod.__grains__ = {'a': ['item', 12], 'c': 8}
        res = grainsmod.set('a', ['item', 14])
        self.assertFalse(res['result'])
        self.assertEqual(res['comment'], 'The key \'a\' exists but is a dict or a list. '
                            + 'Use \'force=True\' to overwrite.')
        self.assertEqual(grainsmod.__grains__, {'a': ['item', 12], 'c': 8})

        # Fails to overwrite a complex value without 'force' in a nested grain
        grainsmod.__grains__ = {'a': 'aval',
                                'b': ['l1', {'l2': ['val1']}],
                                'c': 8}
        res = grainsmod.set('b,l2', 'val2', delimiter=',')
        self.assertFalse(res['result'])
        self.assertEqual(res['comment'], 'The key \'b,l2\' exists but is a dict or a '
                            + 'list. Use \'force=True\' to overwrite.')
        self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                'b': ['l1', {'l2': ['val1']}],
                                                'c': 8})

    def test_set_nested_fails_replace_simple_value(self):
        # Fails to replace a simple value with a new dictionary consisting
        # of the specified key and value
        grainsmod.__grains__ = {'a': 'aval', 'b': 'l1', 'c': 8}
        res = grainsmod.set('b,l3', 'val3', delimiter=',')
        self.assertFalse(res['result'])
        self.assertEqual(res['comment'], 'The key \'b\' value is \'l1\', which is '
                            + 'different from the provided key \'l3\'. '
                            + 'Use \'force=True\' to overwrite.')
        self.assertEqual(grainsmod.__grains__, {'a': 'aval', 'b': 'l1', 'c': 8})

    def test_set_simple_value(self):
        grainsmod.__grains__ = {'a': ['b', 'c'], 'c': 8}
        res = grainsmod.set('b', 'bval')
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': 'bval'})
        self.assertEqual(grainsmod.__grains__, {'a': ['b', 'c'],
                                                'b': 'bval',
                                                'c': 8})

    def test_set_replace_value(self):
        grainsmod.__grains__ = {'a': 'aval', 'c': 8}
        res = grainsmod.set('a', 12)
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'a': 12})
        self.assertEqual(grainsmod.__grains__, {'a': 12, 'c': 8})

    def test_set_None_ok(self):
        grainsmod.__grains__ = {'a': 'aval', 'c': 8}
        res = grainsmod.set('b', None)
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': None})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval', 'b': None, 'c': 8})

    def test_set_None_ok_destructive(self):
        grainsmod.__grains__ = {'a': 'aval', 'c': 8}
        res = grainsmod.set('b', None, destructive=True)
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': None})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval', 'c': 8})

    def test_set_None_replace_ok(self):
        grainsmod.__grains__ = {'a': 'aval', 'c': 8}
        res = grainsmod.set('a', None)
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'a': None})
        self.assertEqual(grainsmod.__grains__, {'a': None, 'c': 8})

    def test_set_None_force_destructive(self):
        grainsmod.__grains__ = {'a': 'aval', 'c': 8}
        res = grainsmod.set('a', None, force=True, destructive=True)
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'a': None})
        self.assertEqual(grainsmod.__grains__, {'c': 8})

    def test_set_replace_value_was_complex_force(self):
        grainsmod.__grains__ = {'a': ['item', 12], 'c': 8}
        res = grainsmod.set('a', 'aval', force=True)
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'a': 'aval'})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval', 'c': 8})

    def test_set_complex_value_force(self):
        grainsmod.__grains__ = {'a': 'aval', 'c': 8}
        res = grainsmod.set('a', ['item', 12], force=True)
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'a': ['item', 12]})
        self.assertEqual(grainsmod.__grains__, {'a': ['item', 12], 'c': 8})

    def test_set_nested_create(self):
        grainsmod.__grains__ = {'a': 'aval', 'c': 8}
        res = grainsmod.set('b,nested', 'val', delimiter=',')
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': {'nested': 'val'}})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                'b': {'nested': 'val'},
                                                'c': 8})

    def test_set_nested_update_dict(self):
        grainsmod.__grains__ = {'a': 'aval', 'b': {'nested': 'val'}, 'c': 8}
        res = grainsmod.set('b,nested', 'val2', delimiter=',')
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': {'nested': 'val2'}})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                'b': {'nested': 'val2'},
                                                'c': 8})

    def test_set_nested_update_dict_remove_key(self):
        grainsmod.__grains__ = {'a': 'aval', 'b': {'nested': 'val'}, 'c': 8}
        res = grainsmod.set('b,nested', None, delimiter=',', destructive=True)
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': {}})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval', 'b': {}, 'c': 8})

    def test_set_nested_update_dict_new_key(self):
        grainsmod.__grains__ = {'a': 'aval', 'b': {'nested': 'val'}, 'c': 8}
        res = grainsmod.set('b,b2', 'val2', delimiter=',')
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': {'b2': 'val2', 'nested': 'val'}})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                'b': {'b2': 'val2',
                                                      'nested': 'val'},
                                                'c': 8})

    def test_set_nested_list_replace_key(self):
        grainsmod.__grains__ = {'a': 'aval', 'b': ['l1', 'l2', 'l3'], 'c': 8}
        res = grainsmod.set('b,l2', 'val2', delimiter=',')
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': ['l1', {'l2': 'val2'}, 'l3']})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                'b': ['l1', {'l2': 'val2'}, 'l3'],
                                                'c': 8})

    def test_set_nested_list_update_dict_key(self):
        grainsmod.__grains__ = {'a': 'aval', 'b': ['l1', {'l2': 'val1'}], 'c': 8}
        res = grainsmod.set('b,l2', 'val2', delimiter=',')
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': ['l1', {'l2': 'val2'}]})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                'b': ['l1', {'l2': 'val2'}],
                                                'c': 8})

    def test_set_nested_list_update_dict_key_overwrite(self):
        grainsmod.__grains__ = {'a': 'aval',
                                'b': ['l1', {'l2': ['val1']}],
                                'c': 8}
        res = grainsmod.set('b,l2', 'val2', delimiter=',', force=True)
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': ['l1', {'l2': 'val2'}]})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                'b': ['l1', {'l2': 'val2'}],
                                                'c': 8})

    def test_set_nested_list_append_dict_key(self):
        grainsmod.__grains__ = {'a': 'aval',
                                'b': ['l1', {'l2': 'val2'}],
                                'c': 8}
        res = grainsmod.set('b,l3', 'val3', delimiter=',')
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': ['l1', {'l2': 'val2'}, {'l3': 'val3'}]})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                'b': ['l1',
                                                      {'l2': 'val2'},
                                                      {'l3': 'val3'}],
                                                'c': 8})

    def test_set_nested_existing_value_is_the_key(self):
        grainsmod.__grains__ = {'a': 'aval', 'b': 'l3', 'c': 8}
        res = grainsmod.set('b,l3', 'val3', delimiter=',')
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': {'l3': 'val3'}})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                'b': {'l3': 'val3'},
                                                'c': 8})

    def test_set_nested_existing_value_overwrite(self):
        grainsmod.__grains__ = {'a': 'aval', 'b': 'l1', 'c': 8}
        res = grainsmod.set('b,l3', 'val3', delimiter=',', force=True)
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': {'l3': 'val3'}})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                'b': {'l3': 'val3'},
                                                'c': 8})

    def test_set_deeply_nested_update(self):
        grainsmod.__grains__ = {'a': 'aval',
                                'b': {'l1': ['l21', 'l22', {'l23': 'l23val'}]},
                                'c': 8}
        res = grainsmod.set('b,l1,l23', 'val', delimiter=',')
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': {'l1': ['l21', 'l22', {'l23': 'val'}]}})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                'b': {'l1': ['l21',
                                                             'l22',
                                                             {'l23': 'val'}]},
                                                'c': 8})

    def test_set_deeply_nested_create(self):
        grainsmod.__grains__ = {'a': 'aval',
                                'b': {'l1': ['l21', 'l22', {'l23': 'l23val'}]},
                                'c': 8}
        res = grainsmod.set('b,l1,l24,l241', 'val', delimiter=',')
        self.assertTrue(res['result'])
        self.assertEqual(res['changes'], {'b': {'l1': ['l21',
                                            'l22',
                                            {'l23': 'l23val'},
                                            {'l24': {'l241': 'val'}}]}})
        self.assertEqual(grainsmod.__grains__, {'a': 'aval',
                                                'b': {'l1': [
                                                    'l21',
                                                    'l22',
                                                    {'l23': 'l23val'},
                                                    {'l24': {'l241': 'val'}}]},
                                                'c': 8})

    def test_get_ordered(self):
        grainsmod.__grains__ = OrderedDict([
                                ('a', 'aval'),
                                ('b', OrderedDict([
                                    ('z', 'zval'),
                                    ('l1', ['l21',
                                            'l22',
                                            OrderedDict([('l23', 'l23val')])])
                                    ])),
                                ('c', 8)])
        res = grainsmod.get('b')
        self.assertEqual(type(res), OrderedDict)
        # Check that order really matters
        self.assertTrue(res == OrderedDict([
                                  ('z', 'zval'),
                                  ('l1', ['l21',
                                          'l22',
                                          OrderedDict([('l23', 'l23val')])]),
                                  ]))
        self.assertFalse(res == OrderedDict([
                                  ('l1', ['l21',
                                          'l22',
                                          OrderedDict([('l23', 'l23val')])]),
                                  ('z', 'zval'),
                                  ]))

    def test_get_unordered(self):
        grainsmod.__grains__ = OrderedDict([
                                ('a', 'aval'),
                                ('b', OrderedDict([
                                    ('z', 'zval'),
                                    ('l1', ['l21',
                                            'l22',
                                            OrderedDict([('l23', 'l23val')])])
                                    ])),
                                ('c', 8)])
        res = grainsmod.get('b', ordered=False)
        self.assertEqual(type(res), dict)
        # Check that order doesn't matter
        self.assertTrue(res == OrderedDict([
                                  ('l1', ['l21',
                                          'l22',
                                          OrderedDict([('l23', 'l23val')])]),
                                  ('z', 'zval'),
                                  ]))


if __name__ == '__main__':
    from integration import run_tests
    run_tests(GrainsModuleTestCase, needs_daemon=False)
